@@ -1,13 +1,3 @@
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (hagberg@lanl.gov),
-#          Pieter Swart (swart@lanl.gov),
-#          Dan Schult(dschult@colgate.edu)
 """
 View Classes provide node, edge and degree "views" of a graph.
 
@@ -92,17 +82,33 @@ EdgeDataView
 
     The argument `nbunch` restricts edges to those incident to nodes in nbunch.
 """
-from collections.abc import Mapping, Set, Iterable
+from collections.abc import Mapping, Set
 import networkx as nx
 
-__all__ = ['NodeView', 'NodeDataView',
-           'EdgeView', 'OutEdgeView', 'InEdgeView',
-           'EdgeDataView', 'OutEdgeDataView', 'InEdgeDataView',
-           'MultiEdgeView', 'OutMultiEdgeView', 'InMultiEdgeView',
-           'MultiEdgeDataView', 'OutMultiEdgeDataView', 'InMultiEdgeDataView',
-           'DegreeView', 'DiDegreeView', 'InDegreeView', 'OutDegreeView',
-           'MultiDegreeView', 'DiMultiDegreeView',
-           'InMultiDegreeView', 'OutMultiDegreeView']
+__all__ = [
+    "NodeView",
+    "NodeDataView",
+    "EdgeView",
+    "OutEdgeView",
+    "InEdgeView",
+    "EdgeDataView",
+    "OutEdgeDataView",
+    "InEdgeDataView",
+    "MultiEdgeView",
+    "OutMultiEdgeView",
+    "InMultiEdgeView",
+    "MultiEdgeDataView",
+    "OutMultiEdgeDataView",
+    "InMultiEdgeDataView",
+    "DegreeView",
+    "DiDegreeView",
+    "InDegreeView",
+    "OutDegreeView",
+    "MultiDegreeView",
+    "DiMultiDegreeView",
+    "InMultiDegreeView",
+    "OutMultiDegreeView",
+]
 
 
 # NodeViews
@@ -124,20 +130,22 @@ class NodeView(Mapping, Set):
     >>> NV = G.nodes()
     >>> 2 in NV
     True
-    >>> for n in NV: print(n)
+    >>> for n in NV:
+    ...     print(n)
     0
     1
     2
-    >>> assert(NV & {1, 2, 3} == {1, 2})
+    >>> assert NV & {1, 2, 3} == {1, 2}
 
-    >>> G.add_node(2, color='blue')
+    >>> G.add_node(2, color="blue")
     >>> NV[2]
     {'color': 'blue'}
-    >>> G.add_node(8, color='red')
+    >>> G.add_node(8, color="red")
     >>> NDV = G.nodes(data=True)
     >>> (2, NV[2]) in NDV
     True
-    >>> for n, dd in NDV: print((n, dd.get('color', 'aqua')))
+    >>> for n, dd in NDV:
+    ...     print((n, dd.get("color", "aqua")))
     (0, 'aqua')
     (1, 'aqua')
     (2, 'blue')
@@ -145,10 +153,11 @@ class NodeView(Mapping, Set):
     >>> NDV[2] == NV[2]
     True
 
-    >>> NVdata = G.nodes(data='color', default='aqua')
+    >>> NVdata = G.nodes(data="color", default="aqua")
     >>> (2, NVdata[2]) in NVdata
     True
-    >>> for n, dd in NVdata: print((n, dd))
+    >>> for n, dd in NVdata:
+    ...     print((n, dd))
     (0, 'aqua')
     (1, 'aqua')
     (2, 'blue')
@@ -156,13 +165,14 @@ class NodeView(Mapping, Set):
     >>> NVdata[2] == NV[2]  # NVdata gets 'color', NV gets datadict
     False
     """
-    __slots__ = '_nodes',
+
+    __slots__ = ("_nodes",)
 
     def __getstate__(self):
-        return {'_nodes': self._nodes}
+        return {"_nodes": self._nodes}
 
     def __setstate__(self, state):
-        self._nodes = state['_nodes']
+        self._nodes = state["_nodes"]
 
     def __init__(self, graph):
         self._nodes = graph._node
@@ -175,6 +185,11 @@ class NodeView(Mapping, Set):
         return iter(self._nodes)
 
     def __getitem__(self, n):
+        if isinstance(n, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.nodes)[{n.start}:{n.stop}:{n.step}]"
+            )
         return self._nodes[n]
 
     # Set methods
@@ -200,7 +215,7 @@ class NodeView(Mapping, Set):
         return str(list(self))
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, tuple(self))
+        return f"{self.__class__.__name__}({tuple(self)})"
 
 
 class NodeDataView(Set):
@@ -220,17 +235,16 @@ class NodeDataView(Set):
     data : bool or string (default=False)
     default : object (default=None)
     """
-    __slots__ = ('_nodes', '_data', '_default')
+
+    __slots__ = ("_nodes", "_data", "_default")
 
     def __getstate__(self):
-        return {'_nodes': self._nodes,
-                '_data': self._data,
-                '_default': self._default}
+        return {"_nodes": self._nodes, "_data": self._data, "_default": self._default}
 
     def __setstate__(self, state):
-        self._nodes = state['_nodes']
-        self._data = state['_data']
-        self._default = state['_default']
+        self._nodes = state["_nodes"]
+        self._data = state["_data"]
+        self._default = state["_default"]
 
     def __init__(self, nodedict, data=False, default=None):
         self._nodes = nodedict
@@ -244,7 +258,7 @@ class NodeDataView(Set):
         except TypeError as err:
             if "unhashable" in str(err):
                 msg = " : Could be b/c data=True or your values are unhashable"
-                raise TypeError(str(err) + msg)
+                raise TypeError(str(err) + msg) from err
             raise
 
     def __len__(self):
@@ -256,8 +270,10 @@ class NodeDataView(Set):
             return iter(self._nodes)
         if data is True:
             return iter(self._nodes.items())
-        return ((n, dd[data] if data in dd else self._default)
-                for n, dd in self._nodes.items())
+        return (
+            (n, dd[data] if data in dd else self._default)
+            for n, dd in self._nodes.items()
+        )
 
     def __contains__(self, n):
         try:
@@ -274,6 +290,11 @@ class NodeDataView(Set):
         return n in self._nodes and self[n] == d
 
     def __getitem__(self, n):
+        if isinstance(n, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.nodes.data())[{n.start}:{n.stop}:{n.step}]"
+            )
         ddict = self._nodes[n]
         data = self._data
         if data is False or data is True:
@@ -284,16 +305,16 @@ class NodeDataView(Set):
         return str(list(self))
 
     def __repr__(self):
+        name = self.__class__.__name__
         if self._data is False:
-            return '%s(%r)' % (self.__class__.__name__, tuple(self))
+            return f"{name}({tuple(self)})"
         if self._data is True:
-            return '%s(%r)' % (self.__class__.__name__, dict(self))
-        return '%s(%r, data=%r)' % \
-               (self.__class__.__name__, dict(self), self._data)
+            return f"{name}({dict(self)})"
+        return f"{name}({dict(self)}, data={self._data!r})"
 
 
 # DegreeViews
-class DiDegreeView(object):
+class DiDegreeView:
     """A View class for degree of nodes in a NetworkX Graph
 
     The functionality is like dict.items() with (node, degree) pairs.
@@ -315,8 +336,8 @@ class DiDegreeView(object):
     --------
     >>> G = nx.path_graph(3)
     >>> DV = G.degree()
-    >>> assert(DV[2] == 1)
-    >>> assert(sum(deg for n, deg in DV) == 4)
+    >>> assert DV[2] == 1
+    >>> assert sum(deg for n, deg in DV) == 4
 
     >>> DVweight = G.degree(weight="span")
     >>> G.add_edge(1, 2, span=34)
@@ -328,15 +349,14 @@ class DiDegreeView(object):
     70
 
     >>> DVnbunch = G.degree(nbunch=(1, 2))
-    >>> assert(len(list(DVnbunch)) == 2)  # iteration over nbunch only
+    >>> assert len(list(DVnbunch)) == 2  # iteration over nbunch only
     """
 
     def __init__(self, G, nbunch=None, weight=None):
         self._graph = G
         self._succ = G._succ if hasattr(G, "_succ") else G._adj
         self._pred = G._pred if hasattr(G, "_pred") else G._adj
-        self._nodes = self._succ if nbunch is None \
-            else list(G.nbunch_iter(nbunch))
+        self._nodes = self._succ if nbunch is None else list(G.nbunch_iter(nbunch))
         self._weight = weight
 
     def __call__(self, nbunch=None, weight=None):
@@ -359,8 +379,9 @@ class DiDegreeView(object):
         preds = self._pred[n]
         if weight is None:
             return len(succs) + len(preds)
-        return sum(dd.get(weight, 1) for dd in succs.values()) + \
-            sum(dd.get(weight, 1) for dd in preds.values())
+        return sum(dd.get(weight, 1) for dd in succs.values()) + sum(
+            dd.get(weight, 1) for dd in preds.values()
+        )
 
     def __iter__(self):
         weight = self._weight
@@ -373,8 +394,9 @@ class DiDegreeView(object):
             for n in self._nodes:
                 succs = self._succ[n]
                 preds = self._pred[n]
-                deg = sum(dd.get(weight, 1) for dd in succs.values()) \
-                    + sum(dd.get(weight, 1) for dd in preds.values())
+                deg = sum(dd.get(weight, 1) for dd in succs.values()) + sum(
+                    dd.get(weight, 1) for dd in preds.values()
+                )
                 yield (n, deg)
 
     def __len__(self):
@@ -384,7 +406,7 @@ class DiDegreeView(object):
         return str(list(self))
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, dict(self))
+        return f"{self.__class__.__name__}({dict(self)})"
 
 
 class DegreeView(DiDegreeView):
@@ -414,9 +436,9 @@ class DegreeView(DiDegreeView):
     --------
     >>> G = nx.path_graph(3)
     >>> DV = G.degree()
-    >>> assert(DV[2] == 1)
-    >>> assert(G.degree[2] == 1)
-    >>> assert(sum(deg for n, deg in DV) == 4)
+    >>> assert DV[2] == 1
+    >>> assert G.degree[2] == 1
+    >>> assert sum(deg for n, deg in DV) == 4
 
     >>> DVweight = G.degree(weight="span")
     >>> G.add_edge(1, 2, span=34)
@@ -428,7 +450,7 @@ class DegreeView(DiDegreeView):
     70
 
     >>> DVnbunch = G.degree(nbunch=(1, 2))
-    >>> assert(len(list(DVnbunch)) == 2)  # iteration over nbunch only
+    >>> assert len(list(DVnbunch)) == 2  # iteration over nbunch only
     """
 
     def __getitem__(self, n):
@@ -436,8 +458,9 @@ class DegreeView(DiDegreeView):
         nbrs = self._succ[n]
         if weight is None:
             return len(nbrs) + (n in nbrs)
-        return sum(dd.get(weight, 1) for dd in nbrs.values()) + \
-            (n in nbrs and nbrs[n].get(weight, 1))
+        return sum(dd.get(weight, 1) for dd in nbrs.values()) + (
+            n in nbrs and nbrs[n].get(weight, 1)
+        )
 
     def __iter__(self):
         weight = self._weight
@@ -448,8 +471,9 @@ class DegreeView(DiDegreeView):
         else:
             for n in self._nodes:
                 nbrs = self._succ[n]
-                deg = sum(dd.get(weight, 1) for dd in nbrs.values()) + \
-                    (n in nbrs and nbrs[n].get(weight, 1))
+                deg = sum(dd.get(weight, 1) for dd in nbrs.values()) + (
+                    n in nbrs and nbrs[n].get(weight, 1)
+                )
                 yield (n, deg)
 
 
@@ -506,11 +530,13 @@ class MultiDegreeView(DiDegreeView):
         weight = self._weight
         nbrs = self._succ[n]
         if weight is None:
-            return sum(len(keys) for keys in nbrs.values()) + \
-                (n in nbrs and len(nbrs[n]))
+            return sum(len(keys) for keys in nbrs.values()) + (
+                n in nbrs and len(nbrs[n])
+            )
         # edge weighted graph - degree is sum of nbr edge weights
-        deg = sum(d.get(weight, 1) for key_dict in nbrs.values()
-                  for d in key_dict.values())
+        deg = sum(
+            d.get(weight, 1) for key_dict in nbrs.values() for d in key_dict.values()
+        )
         if n in nbrs:
             deg += sum(d.get(weight, 1) for d in nbrs[n].values())
         return deg
@@ -520,14 +546,18 @@ class MultiDegreeView(DiDegreeView):
         if weight is None:
             for n in self._nodes:
                 nbrs = self._succ[n]
-                deg = sum(len(keys) for keys in nbrs.values()) + \
-                    (n in nbrs and len(nbrs[n]))
+                deg = sum(len(keys) for keys in nbrs.values()) + (
+                    n in nbrs and len(nbrs[n])
+                )
                 yield (n, deg)
         else:
             for n in self._nodes:
                 nbrs = self._succ[n]
-                deg = sum(d.get(weight, 1) for key_dict in nbrs.values()
-                          for d in key_dict.values())
+                deg = sum(
+                    d.get(weight, 1)
+                    for key_dict in nbrs.values()
+                    for d in key_dict.values()
+                )
                 if n in nbrs:
                     deg += sum(d.get(weight, 1) for d in nbrs[n].values())
                 yield (n, deg)
@@ -541,13 +571,15 @@ class DiMultiDegreeView(DiDegreeView):
         succs = self._succ[n]
         preds = self._pred[n]
         if weight is None:
-            return sum(len(keys) for keys in succs.values()) + \
-                sum(len(keys) for keys in preds.values())
+            return sum(len(keys) for keys in succs.values()) + sum(
+                len(keys) for keys in preds.values()
+            )
         # edge weighted graph - degree is sum of nbr edge weights
-        deg = sum(d.get(weight, 1) for key_dict in succs.values()
-                  for d in key_dict.values()) + \
-            sum(d.get(weight, 1) for key_dict in preds.values()
-                for d in key_dict.values())
+        deg = sum(
+            d.get(weight, 1) for key_dict in succs.values() for d in key_dict.values()
+        ) + sum(
+            d.get(weight, 1) for key_dict in preds.values() for d in key_dict.values()
+        )
         return deg
 
     def __iter__(self):
@@ -556,17 +588,23 @@ class DiMultiDegreeView(DiDegreeView):
             for n in self._nodes:
                 succs = self._succ[n]
                 preds = self._pred[n]
-                deg = sum(len(keys) for keys in succs.values()) + \
-                    sum(len(keys) for keys in preds.values())
+                deg = sum(len(keys) for keys in succs.values()) + sum(
+                    len(keys) for keys in preds.values()
+                )
                 yield (n, deg)
         else:
             for n in self._nodes:
                 succs = self._succ[n]
                 preds = self._pred[n]
-                deg = sum(d.get(weight, 1) for key_dict in succs.values()
-                          for d in key_dict.values()) + \
-                    sum(d.get(weight, 1) for key_dict in preds.values()
-                        for d in key_dict.values())
+                deg = sum(
+                    d.get(weight, 1)
+                    for key_dict in succs.values()
+                    for d in key_dict.values()
+                ) + sum(
+                    d.get(weight, 1)
+                    for key_dict in preds.values()
+                    for d in key_dict.values()
+                )
                 yield (n, deg)
 
 
@@ -579,8 +617,9 @@ class InMultiDegreeView(DiDegreeView):
         if weight is None:
             return sum(len(data) for data in nbrs.values())
         # edge weighted graph - degree is sum of nbr edge weights
-        return sum(d.get(weight, 1) for key_dict in nbrs.values()
-                   for d in key_dict.values())
+        return sum(
+            d.get(weight, 1) for key_dict in nbrs.values() for d in key_dict.values()
+        )
 
     def __iter__(self):
         weight = self._weight
@@ -592,8 +631,11 @@ class InMultiDegreeView(DiDegreeView):
         else:
             for n in self._nodes:
                 nbrs = self._pred[n]
-                deg = sum(d.get(weight, 1) for key_dict in nbrs.values()
-                          for d in key_dict.values())
+                deg = sum(
+                    d.get(weight, 1)
+                    for key_dict in nbrs.values()
+                    for d in key_dict.values()
+                )
                 yield (n, deg)
 
 
@@ -606,8 +648,9 @@ class OutMultiDegreeView(DiDegreeView):
         if weight is None:
             return sum(len(data) for data in nbrs.values())
         # edge weighted graph - degree is sum of nbr edge weights
-        return sum(d.get(weight, 1) for key_dict in nbrs.values()
-                   for d in key_dict.values())
+        return sum(
+            d.get(weight, 1) for key_dict in nbrs.values() for d in key_dict.values()
+        )
 
     def __iter__(self):
         weight = self._weight
@@ -619,34 +662,48 @@ class OutMultiDegreeView(DiDegreeView):
         else:
             for n in self._nodes:
                 nbrs = self._succ[n]
-                deg = sum(d.get(weight, 1) for key_dict in nbrs.values()
-                          for d in key_dict.values())
+                deg = sum(
+                    d.get(weight, 1)
+                    for key_dict in nbrs.values()
+                    for d in key_dict.values()
+                )
                 yield (n, deg)
 
 
 # EdgeDataViews
-class OutEdgeDataView(object):
+class OutEdgeDataView:
     """EdgeDataView for outward edges of DiGraph; See EdgeDataView"""
-    __slots__ = ('_viewer', '_nbunch', '_data', '_default',
-                 '_adjdict', '_nodes_nbrs', '_report')
+
+    __slots__ = (
+        "_viewer",
+        "_nbunch",
+        "_data",
+        "_default",
+        "_adjdict",
+        "_nodes_nbrs",
+        "_report",
+    )
 
     def __getstate__(self):
-        return {'viewer': self._viewer,
-                'nbunch': self._nbunch,
-                'data': self._data,
-                'default': self._default}
+        return {
+            "viewer": self._viewer,
+            "nbunch": self._nbunch,
+            "data": self._data,
+            "default": self._default,
+        }
 
     def __setstate__(self, state):
         self.__init__(**state)
 
     def __init__(self, viewer, nbunch=None, data=False, default=None):
         self._viewer = viewer
-        self._adjdict = viewer._adjdict
+        adjdict = self._adjdict = viewer._adjdict
         if nbunch is None:
-            self._nodes_nbrs = self._adjdict.items
+            self._nodes_nbrs = adjdict.items
         else:
-            nbunch = list(viewer._graph.nbunch_iter(nbunch))
-            self._nodes_nbrs = lambda: [(n, self._adjdict[n]) for n in nbunch]
+            # dict retains order of nodes but acts like a set
+            nbunch = dict.fromkeys(viewer._graph.nbunch_iter(nbunch))
+            self._nodes_nbrs = lambda: [(n, adjdict[n]) for n in nbunch]
         self._nbunch = nbunch
         self._data = data
         self._default = default
@@ -656,19 +713,27 @@ class OutEdgeDataView(object):
         elif data is False:
             self._report = lambda n, nbr, dd: (n, nbr)
         else:  # data is attribute name
-            self._report = lambda n, nbr, dd: \
-                (n, nbr, dd[data]) if data in dd else (n, nbr, default)
+            self._report = (
+                lambda n, nbr, dd: (n, nbr, dd[data])
+                if data in dd
+                else (n, nbr, default)
+            )
 
     def __len__(self):
         return sum(len(nbrs) for n, nbrs in self._nodes_nbrs())
 
     def __iter__(self):
-        return (self._report(n, nbr, dd) for n, nbrs in self._nodes_nbrs()
-                for nbr, dd in nbrs.items())
+        return (
+            self._report(n, nbr, dd)
+            for n, nbrs in self._nodes_nbrs()
+            for nbr, dd in nbrs.items()
+        )
 
     def __contains__(self, e):
+        u, v = e[:2]
+        if self._nbunch is not None and u not in self._nbunch:
+            return False  # this edge doesn't start in nbunch
         try:
-            u, v = e[:2]
             ddict = self._adjdict[u][v]
         except KeyError:
             return False
@@ -678,7 +743,7 @@ class OutEdgeDataView(object):
         return str(list(self))
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, list(self))
+        return f"{self.__class__.__name__}({list(self)})"
 
 
 class EdgeDataView(OutEdgeDataView):
@@ -704,11 +769,12 @@ class EdgeDataView(OutEdgeDataView):
     Examples
     --------
     >>> G = nx.path_graph(3)
-    >>> G.add_edge(1, 2, foo='bar')
-    >>> list(G.edges(data='foo', default='biz'))
+    >>> G.add_edge(1, 2, foo="bar")
+    >>> list(G.edges(data="foo", default="biz"))
     [(0, 1, 'biz'), (1, 2, 'bar')]
-    >>> assert((0, 1, 'biz') in G.edges(data='foo', default='biz'))
+    >>> assert (0, 1, "biz") in G.edges(data="foo", default="biz")
     """
+
     __slots__ = ()
 
     def __len__(self):
@@ -724,28 +790,33 @@ class EdgeDataView(OutEdgeDataView):
         del seen
 
     def __contains__(self, e):
+        u, v = e[:2]
+        if self._nbunch is not None and u not in self._nbunch and v not in self._nbunch:
+            return False  # this edge doesn't start and it doesn't end in nbunch
         try:
-            u, v = e[:2]
             ddict = self._adjdict[u][v]
         except KeyError:
-            try:
-                ddict = self._adjdict[v][u]
-            except KeyError:
-                return False
+            return False
         return e == self._report(u, v, ddict)
 
 
 class InEdgeDataView(OutEdgeDataView):
     """An EdgeDataView class for outward edges of DiGraph; See EdgeDataView"""
+
     __slots__ = ()
 
     def __iter__(self):
-        return (self._report(nbr, n, dd) for n, nbrs in self._nodes_nbrs()
-                for nbr, dd in nbrs.items())
+        return (
+            self._report(nbr, n, dd)
+            for n, nbrs in self._nodes_nbrs()
+            for nbr, dd in nbrs.items()
+        )
 
     def __contains__(self, e):
+        u, v = e[:2]
+        if self._nbunch is not None and v not in self._nbunch:
+            return False  # this edge doesn't end in nbunch
         try:
-            u, v = e[:2]
             ddict = self._adjdict[v][u]
         except KeyError:
             return False
@@ -754,28 +825,31 @@ class InEdgeDataView(OutEdgeDataView):
 
 class OutMultiEdgeDataView(OutEdgeDataView):
     """An EdgeDataView for outward edges of MultiDiGraph; See EdgeDataView"""
-    __slots__ = ('keys',)
+
+    __slots__ = ("keys",)
 
     def __getstate__(self):
-        return {'viewer': self._viewer,
-                'nbunch': self._nbunch,
-                'keys': self.keys,
-                'data': self._data,
-                'default': self._default}
+        return {
+            "viewer": self._viewer,
+            "nbunch": self._nbunch,
+            "keys": self.keys,
+            "data": self._data,
+            "default": self._default,
+        }
 
     def __setstate__(self, state):
         self.__init__(**state)
 
-    def __init__(self, viewer, nbunch=None,
-                 data=False, keys=False, default=None):
+    def __init__(self, viewer, nbunch=None, data=False, keys=False, default=None):
         self._viewer = viewer
-        self._adjdict = viewer._adjdict
+        adjdict = self._adjdict = viewer._adjdict
         self.keys = keys
         if nbunch is None:
-            self._nodes_nbrs = self._adjdict.items
+            self._nodes_nbrs = adjdict.items
         else:
-            nbunch = list(viewer._graph.nbunch_iter(nbunch))
-            self._nodes_nbrs = lambda: [(n, self._adjdict[n]) for n in nbunch]
+            # dict retains order of nodes but acts like a set
+            nbunch = dict.fromkeys(viewer._graph.nbunch_iter(nbunch))
+            self._nodes_nbrs = lambda: [(n, adjdict[n]) for n in nbunch]
         self._nbunch = nbunch
         self._data = data
         self._default = default
@@ -792,21 +866,33 @@ class OutMultiEdgeDataView(OutEdgeDataView):
                 self._report = lambda n, nbr, k, dd: (n, nbr)
         else:  # data is attribute name
             if keys is True:
-                self._report = lambda n, nbr, k, dd: (n, nbr, k, dd[data]) \
-                    if data in dd else (n, nbr, k, default)
+                self._report = (
+                    lambda n, nbr, k, dd: (n, nbr, k, dd[data])
+                    if data in dd
+                    else (n, nbr, k, default)
+                )
             else:
-                self._report = lambda n, nbr, k, dd: (n, nbr, dd[data]) \
-                    if data in dd else (n, nbr, default)
+                self._report = (
+                    lambda n, nbr, k, dd: (n, nbr, dd[data])
+                    if data in dd
+                    else (n, nbr, default)
+                )
 
     def __len__(self):
         return sum(1 for e in self)
 
     def __iter__(self):
-        return (self._report(n, nbr, k, dd) for n, nbrs in self._nodes_nbrs()
-                for nbr, kd in nbrs.items() for k, dd in kd.items())
+        return (
+            self._report(n, nbr, k, dd)
+            for n, nbrs in self._nodes_nbrs()
+            for nbr, kd in nbrs.items()
+            for k, dd in kd.items()
+        )
 
     def __contains__(self, e):
         u, v = e[:2]
+        if self._nbunch is not None and u not in self._nbunch:
+            return False  # this edge doesn't start in nbunch
         try:
             kdict = self._adjdict[u][v]
         except KeyError:
@@ -826,6 +912,7 @@ class OutMultiEdgeDataView(OutEdgeDataView):
 
 class MultiEdgeDataView(OutMultiEdgeDataView):
     """An EdgeDataView class for edges of MultiGraph; See EdgeDataView"""
+
     __slots__ = ()
 
     def __iter__(self):
@@ -840,6 +927,8 @@ class MultiEdgeDataView(OutMultiEdgeDataView):
 
     def __contains__(self, e):
         u, v = e[:2]
+        if self._nbunch is not None and u not in self._nbunch and v not in self._nbunch:
+            return False  # this edge doesn't start and doesn't end in nbunch
         try:
             kdict = self._adjdict[u][v]
         except KeyError:
@@ -862,14 +951,21 @@ class MultiEdgeDataView(OutMultiEdgeDataView):
 
 class InMultiEdgeDataView(OutMultiEdgeDataView):
     """An EdgeDataView for inward edges of MultiDiGraph; See EdgeDataView"""
+
     __slots__ = ()
 
     def __iter__(self):
-        return (self._report(nbr, n, k, dd) for n, nbrs in self._nodes_nbrs()
-                for nbr, kd in nbrs.items() for k, dd in kd.items())
+        return (
+            self._report(nbr, n, k, dd)
+            for n, nbrs in self._nodes_nbrs()
+            for nbr, kd in nbrs.items()
+            for k, dd in kd.items()
+        )
 
     def __contains__(self, e):
         u, v = e[:2]
+        if self._nbunch is not None and v not in self._nbunch:
+            return False  # this edge doesn't end in nbunch
         try:
             kdict = self._adjdict[v][u]
         except KeyError:
@@ -887,13 +983,14 @@ class InMultiEdgeDataView(OutMultiEdgeDataView):
 # EdgeViews    have set operations and no data reported
 class OutEdgeView(Set, Mapping):
     """A EdgeView class for outward edges of a DiGraph"""
-    __slots__ = ('_adjdict', '_graph', '_nodes_nbrs')
+
+    __slots__ = ("_adjdict", "_graph", "_nodes_nbrs")
 
     def __getstate__(self):
-        return {'_graph': self._graph}
+        return {"_graph": self._graph}
 
     def __setstate__(self, state):
-        self._graph = G = state['_graph']
+        self._graph = G = state["_graph"]
         self._adjdict = G._succ if hasattr(G, "succ") else G._adj
         self._nodes_nbrs = self._adjdict.items
 
@@ -926,6 +1023,11 @@ class OutEdgeView(Set, Mapping):
 
     # Mapping Methods
     def __getitem__(self, e):
+        if isinstance(e, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.edges)[{e.start}:{e.stop}:{e.step}]"
+            )
         u, v = e
         return self._adjdict[u][v]
 
@@ -945,7 +1047,7 @@ class OutEdgeView(Set, Mapping):
         return str(list(self))
 
     def __repr__(self):
-        return "{0.__class__.__name__}({1!r})".format(self, list(self))
+        return f"{self.__class__.__name__}({list(self)})"
 
 
 class EdgeView(OutEdgeView):
@@ -982,38 +1084,43 @@ class EdgeView(OutEdgeView):
     >>> EV = G.edges()
     >>> (2, 3) in EV
     True
-    >>> for u, v in EV: print((u, v))
+    >>> for u, v in EV:
+    ...     print((u, v))
     (0, 1)
     (1, 2)
     (2, 3)
-    >>> assert(EV & {(1, 2), (3, 4)} == {(1, 2)})
+    >>> assert EV & {(1, 2), (3, 4)} == {(1, 2)}
 
-    >>> EVdata = G.edges(data='color', default='aqua')
-    >>> G.add_edge(2, 3, color='blue')
-    >>> assert((2, 3, 'blue') in EVdata)
-    >>> for u, v, c in EVdata: print("({}, {}) has color: {}".format(u, v, c))
+    >>> EVdata = G.edges(data="color", default="aqua")
+    >>> G.add_edge(2, 3, color="blue")
+    >>> assert (2, 3, "blue") in EVdata
+    >>> for u, v, c in EVdata:
+    ...     print(f"({u}, {v}) has color: {c}")
     (0, 1) has color: aqua
     (1, 2) has color: aqua
     (2, 3) has color: blue
 
     >>> EVnbunch = G.edges(nbunch=2)
-    >>> assert((2, 3) in EVnbunch)
-    >>> assert((0, 1) in EVnbunch)   #  nbunch is ignored in __contains__
-    >>> for u, v in EVnbunch: assert(u == 2 or v == 2)
+    >>> assert (2, 3) in EVnbunch
+    >>> assert (0, 1) not in EVnbunch
+    >>> for u, v in EVnbunch:
+    ...     assert u == 2 or v == 2
 
     >>> MG = nx.path_graph(4, create_using=nx.MultiGraph)
     >>> EVmulti = MG.edges(keys=True)
     >>> (2, 3, 0) in EVmulti
     True
-    >>> (2, 3) in EVmulti   # 2-tuples work even when keys is True
+    >>> (2, 3) in EVmulti  # 2-tuples work even when keys is True
     True
     >>> key = MG.add_edge(2, 3)
-    >>> for u, v, k in EVmulti: print((u, v, k))
+    >>> for u, v, k in EVmulti:
+    ...     print((u, v, k))
     (0, 1, 0)
     (1, 2, 0)
     (2, 3, 0)
     (2, 3, 1)
     """
+
     __slots__ = ()
 
     dataview = EdgeDataView
@@ -1025,7 +1132,7 @@ class EdgeView(OutEdgeView):
     def __iter__(self):
         seen = {}
         for n, nbrs in self._nodes_nbrs():
-            for nbr in nbrs:
+            for nbr in list(nbrs):
                 if nbr not in seen:
                     yield (n, nbr)
             seen[n] = 1
@@ -1041,10 +1148,11 @@ class EdgeView(OutEdgeView):
 
 class InEdgeView(OutEdgeView):
     """A EdgeView class for inward edges of a DiGraph"""
+
     __slots__ = ()
 
     def __setstate__(self, state):
-        self._graph = G = state['_graph']
+        self._graph = G = state["_graph"]
         self._adjdict = G._pred if hasattr(G, "pred") else G._adj
         self._nodes_nbrs = self._adjdict.items
 
@@ -1068,19 +1176,26 @@ class InEdgeView(OutEdgeView):
             return False
 
     def __getitem__(self, e):
+        if isinstance(e, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.in_edges)[{e.start}:{e.stop}:{e.step}]"
+            )
         u, v = e
         return self._adjdict[v][u]
 
 
 class OutMultiEdgeView(OutEdgeView):
     """A EdgeView class for outward edges of a MultiDiGraph"""
+
     __slots__ = ()
 
     dataview = OutMultiEdgeDataView
 
     def __len__(self):
-        return sum(len(kdict) for n, nbrs in self._nodes_nbrs()
-                   for nbr, kdict in nbrs.items())
+        return sum(
+            len(kdict) for n, nbrs in self._nodes_nbrs() for nbr, kdict in nbrs.items()
+        )
 
     def __iter__(self):
         for n, nbrs in self._nodes_nbrs():
@@ -1103,6 +1218,11 @@ class OutMultiEdgeView(OutEdgeView):
             return False
 
     def __getitem__(self, e):
+        if isinstance(e, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.edges)[{e.start}:{e.stop}:{e.step}]"
+            )
         u, v, k = e
         return self._adjdict[u][v][k]
 
@@ -1119,6 +1239,7 @@ class OutMultiEdgeView(OutEdgeView):
 
 class MultiEdgeView(OutMultiEdgeView):
     """A EdgeView class for edges of a MultiGraph"""
+
     __slots__ = ()
 
     dataview = MultiEdgeDataView
@@ -1139,10 +1260,11 @@ class MultiEdgeView(OutMultiEdgeView):
 
 class InMultiEdgeView(OutMultiEdgeView):
     """A EdgeView class for inward edges of a MultiDiGraph"""
+
     __slots__ = ()
 
     def __setstate__(self, state):
-        self._graph = G = state['_graph']
+        self._graph = G = state["_graph"]
         self._adjdict = G._pred if hasattr(G, "pred") else G._adj
         self._nodes_nbrs = self._adjdict.items
 
@@ -1174,5 +1296,10 @@ class InMultiEdgeView(OutMultiEdgeView):
             return False
 
     def __getitem__(self, e):
+        if isinstance(e, slice):
+            raise nx.NetworkXError(
+                f"{type(self).__name__} does not support slicing, "
+                f"try list(G.in_edges)[{e.start}:{e.stop}:{e.step}]"
+            )
         u, v, k = e
         return self._adjdict[v][u][k]
