@@ -1,58 +1,51 @@
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (hagberg@lanl.gov)
-#          Pieter Swart (swart@lanl.gov)
 """Generators for some classic graphs.
 
-The typical graph generator is called as follows:
+The typical graph builder function is called as follows:
 
 >>> G = nx.complete_graph(100)
 
 returning the complete graph on n nodes labeled 0, .., 99
-as a simple graph. Except for empty_graph, all the generators
+as a simple graph. Except for `empty_graph`, all the functions
 in this module return a Graph class (i.e. a simple, undirected graph).
 
 """
-from __future__ import division
 
 import itertools
 
 import networkx as nx
 from networkx.classes import Graph
 from networkx.exception import NetworkXError
-from networkx.utils import accumulate
 from networkx.utils import nodes_or_number
 from networkx.utils import pairwise
 
-__all__ = ['balanced_tree',
-           'barbell_graph',
-           'complete_graph',
-           'complete_multipartite_graph',
-           'circular_ladder_graph',
-           'circulant_graph',
-           'cycle_graph',
-           'dorogovtsev_goltsev_mendes_graph',
-           'empty_graph',
-           'full_rary_tree',
-           'ladder_graph',
-           'lollipop_graph',
-           'null_graph',
-           'path_graph',
-           'star_graph',
-           'trivial_graph',
-           'turan_graph',
-           'wheel_graph']
+__all__ = [
+    "balanced_tree",
+    "barbell_graph",
+    "binomial_tree",
+    "complete_graph",
+    "complete_multipartite_graph",
+    "circular_ladder_graph",
+    "circulant_graph",
+    "cycle_graph",
+    "dorogovtsev_goltsev_mendes_graph",
+    "empty_graph",
+    "full_rary_tree",
+    "ladder_graph",
+    "lollipop_graph",
+    "null_graph",
+    "path_graph",
+    "star_graph",
+    "trivial_graph",
+    "turan_graph",
+    "wheel_graph",
+]
 
 
 # -------------------------------------------------------------------
 #   Some Classic Graphs
 # -------------------------------------------------------------------
 
+
 def _tree_edges(n, r):
     if n == 0:
         return
@@ -72,10 +65,10 @@ def _tree_edges(n, r):
 
 
 def full_rary_tree(r, n, create_using=None):
-    """Creates a full r-ary tree of n vertices.
+    """Creates a full r-ary tree of `n` nodes.
 
     Sometimes called a k-ary, n-ary, or m-ary tree.
-    "... all non-leaf vertices have exactly r children and all levels
+    "... all non-leaf nodes have exactly r children and all levels
     are full except for some rightmost position of the bottom level
     (if a leaf at the bottom level is missing, then so are all of the
     leaves to its right." [1]_
@@ -171,11 +164,9 @@ def barbell_graph(m1, m2, create_using=None):
 
     """
     if m1 < 2:
-        raise NetworkXError(
-            "Invalid graph description, m1 should be >=2")
+        raise NetworkXError("Invalid graph description, m1 should be >=2")
     if m2 < 0:
-        raise NetworkXError(
-            "Invalid graph description, m2 should be >=0")
+        raise NetworkXError("Invalid graph description, m2 should be >=0")
 
     # left barbell
     G = complete_graph(m1, create_using)
@@ -187,8 +178,9 @@ def barbell_graph(m1, m2, create_using=None):
     if m2 > 1:
         G.add_edges_from(pairwise(range(m1, m1 + m2)))
     # right barbell
-    G.add_edges_from((u, v) for u in range(m1 + m2, 2 * m1 + m2)
-                     for v in range(u + 1, 2 * m1 + m2))
+    G.add_edges_from(
+        (u, v) for u in range(m1 + m2, 2 * m1 + m2) for v in range(u + 1, 2 * m1 + m2)
+    )
     # connect it up
     G.add_edge(m1 - 1, m1)
     if m2 > 0:
@@ -196,9 +188,45 @@ def barbell_graph(m1, m2, create_using=None):
     return G
 
 
+def binomial_tree(n, create_using=None):
+    """Returns the Binomial Tree of order n.
+
+    The binomial tree of order 0 consists of a single node. A binomial tree of order k
+    is defined recursively by linking two binomial trees of order k-1: the root of one is
+    the leftmost child of the root of the other.
+
+    Parameters
+    ----------
+    n : int
+        Order of the binomial tree.
+
+    create_using : NetworkX graph constructor, optional (default=nx.Graph)
+       Graph type to create. If graph instance, then cleared before populated.
+
+    Returns
+    -------
+    G : NetworkX graph
+        A binomial tree of $2^n$ nodes and $2^n - 1$ edges.
+
+    """
+    G = nx.empty_graph(1, create_using)
+
+    N = 1
+    for i in range(n):
+        # Use G.edges() to ensure 2-tuples. G.edges is 3-tuple for MultiGraph
+        edges = [(u + N, v + N) for (u, v) in G.edges()]
+        G.add_edges_from(edges)
+        G.add_edge(0, N)
+        N *= 2
+    return G
+
+
 @nodes_or_number(0)
 def complete_graph(n, create_using=None):
-    """ Return the complete graph `K_n` with n nodes.
+    """Return the complete graph `K_n` with n nodes.
+
+    A complete graph on `n` nodes means that all pairs
+    of distinct nodes have an edge connecting them.
 
     Parameters
     ----------
@@ -250,45 +278,64 @@ def circular_ladder_graph(n, create_using=None):
 
 
 def circulant_graph(n, offsets, create_using=None):
-    """Generates the circulant graph $Ci_n(x_1, x_2, ..., x_m)$ with $n$ vertices.
+    r"""Returns the circulant graph $Ci_n(x_1, x_2, ..., x_m)$ with $n$ nodes.
 
-    Returns
-    -------
-    The graph $Ci_n(x_1, ..., x_m)$ consisting of $n$ vertices $0, ..., n-1$ such
-    that the vertex with label $i$ is connected to the vertices labelled $(i + x)$
-    and $(i - x)$, for all $x$ in $x_1$ up to $x_m$, with the indices taken modulo $n$.
+    The circulant graph $Ci_n(x_1, ..., x_m)$ consists of $n$ nodes $0, ..., n-1$
+    such that node $i$ is connected to nodes $(i + x) \mod n$ and $(i - x) \mod n$
+    for all $x$ in $x_1, ..., x_m$. Thus $Ci_n(1)$ is a cycle graph.
 
     Parameters
     ----------
     n : integer
-        The number of vertices the generated graph is to contain.
+        The number of nodes in the graph.
     offsets : list of integers
-        A list of vertex offsets, $x_1$ up to $x_m$, as described above.
+        A list of node offsets, $x_1$ up to $x_m$, as described above.
     create_using : NetworkX graph constructor, optional (default=nx.Graph)
        Graph type to create. If graph instance, then cleared before populated.
 
+    Returns
+    -------
+    NetworkX Graph of type create_using
+
     Examples
     --------
     Many well-known graph families are subfamilies of the circulant graphs;
-    for example, to generate the cycle graph on n points, we connect every
-    vertex to every other at offset plus or minus one. For n = 10,
+    for example, to create the cycle graph on n points, we connect every
+    node to nodes on either side (with offset plus or minus one). For n = 10,
 
-    >>> import networkx
-    >>> G = networkx.generators.classic.circulant_graph(10, [1])
+    >>> G = nx.circulant_graph(10, [1])
     >>> edges = [
-    ...     (0, 9), (0, 1), (1, 2), (2, 3), (3, 4),
-    ...     (4, 5), (5, 6), (6, 7), (7, 8), (8, 9)]
+    ...     (0, 9),
+    ...     (0, 1),
+    ...     (1, 2),
+    ...     (2, 3),
+    ...     (3, 4),
+    ...     (4, 5),
+    ...     (5, 6),
+    ...     (6, 7),
+    ...     (7, 8),
+    ...     (8, 9),
+    ... ]
     ...
     >>> sorted(edges) == sorted(G.edges())
     True
 
-    Similarly, we can generate the complete graph on 5 points with the set of
-    offsets [1, 2]:
+    Similarly, we can create the complete graph
+    on 5 points with the set of offsets [1, 2]:
 
-    >>> G = networkx.generators.classic.circulant_graph(5, [1, 2])
+    >>> G = nx.circulant_graph(5, [1, 2])
     >>> edges = [
-    ...     (0, 1), (0, 2), (0, 3), (0, 4), (1, 2),
-    ...     (1, 3), (1, 4), (2, 3), (2, 4), (3, 4)]
+    ...     (0, 1),
+    ...     (0, 2),
+    ...     (0, 3),
+    ...     (0, 4),
+    ...     (1, 2),
+    ...     (1, 3),
+    ...     (1, 4),
+    ...     (2, 3),
+    ...     (2, 4),
+    ...     (3, 4),
+    ... ]
     ...
     >>> sorted(edges) == sorted(G.edges())
     True
@@ -344,7 +391,7 @@ def dorogovtsev_goltsev_mendes_graph(n, create_using=None):
     G.add_edge(0, 1)
     if n == 0:
         return G
-    new_node = 2         # next node to be added
+    new_node = 2  # next node to be added
     for i in range(1, n + 1):  # iterate over number of generations.
         last_generation_edges = list(G.edges())
         number_of_edges_in_last_generation = len(last_generation_edges)
@@ -356,7 +403,7 @@ def dorogovtsev_goltsev_mendes_graph(n, create_using=None):
 
 
 @nodes_or_number(0)
-def empty_graph(n=0, create_using=None, default=nx.Graph):
+def empty_graph(n=0, create_using=None, default=Graph):
     """Returns the empty graph with n nodes and zero edges.
 
     Parameters
@@ -435,7 +482,7 @@ def empty_graph(n=0, create_using=None, default=nx.Graph):
     """
     if create_using is None:
         G = default()
-    elif hasattr(create_using, '_adj'):
+    elif hasattr(create_using, "_adj"):
         # create_using is a NetworkX style Graph
         create_using.clear()
         G = create_using
@@ -499,11 +546,9 @@ def lollipop_graph(m, n, create_using=None):
     if isinstance(m, int):
         n_nodes = [len(m_nodes) + i for i in n_nodes]
     if M < 2:
-        raise NetworkXError(
-            "Invalid graph description, m should be >=2")
+        raise NetworkXError("Invalid graph description, m should be >=2")
     if N < 0:
-        raise NetworkXError(
-            "Invalid graph description, n should be >=0")
+        raise NetworkXError("Invalid graph description, n should be >=0")
 
     # the ball
     G = complete_graph(m_nodes, create_using)
@@ -536,8 +581,8 @@ def path_graph(n, create_using=None):
     Parameters
     ----------
     n : int or iterable
-        If an integer, node labels are 0 to n with center 0.
-        If an iterable of nodes, the center is the first.
+        If an integer, nodes are 0 to n - 1.
+        If an iterable of nodes, in the order they appear in the path.
     create_using : NetworkX graph constructor, optional (default=nx.Graph)
        Graph type to create. If graph instance, then cleared before populated.
 
@@ -550,7 +595,7 @@ def path_graph(n, create_using=None):
 
 @nodes_or_number(0)
 def star_graph(n, create_using=None):
-    """ Return the star graph
+    """Return the star graph
 
     The star graph consists of one center node connected to n outer nodes.
 
@@ -579,28 +624,26 @@ def star_graph(n, create_using=None):
 
 
 def trivial_graph(create_using=None):
-    """ Return the Trivial graph with one node (with label 0) and no edges.
-
-    """
+    """Return the Trivial graph with one node (with label 0) and no edges."""
     G = empty_graph(1, create_using)
     return G
 
 
 def turan_graph(n, r):
-    r""" Return the Turan Graph
+    r"""Return the Turan Graph
 
-    The Turan Graph is a complete multipartite graph on $n$ vertices
-    with $r$ disjoint subsets. It is the graph with the edges for any graph with
-    $n$ vertices and $r$ disjoint subsets.
+    The Turan Graph is a complete multipartite graph on $n$ nodes
+    with $r$ disjoint subsets. That is, edges connect each node to
+    every node not in its subset.
 
-    Given $n$ and $r$, we generate a complete multipartite graph with
+    Given $n$ and $r$, we create a complete multipartite graph with
     $r-(n \mod r)$ partitions of size $n/r$, rounded down, and
     $n \mod r$ partitions of size $n/r+1$, rounded down.
 
     Parameters
     ----------
     n : int
-        The number of vertices.
+        The number of nodes.
     r : int
         The number of partitions.
         Must be less than or equal to n.
@@ -621,7 +664,7 @@ def turan_graph(n, r):
 
 @nodes_or_number(0)
 def wheel_graph(n, create_using=None):
-    """ Return the wheel graph
+    """Return the wheel graph
 
     The wheel graph consists of a hub node connected to a cycle of (n-1) nodes.
 
@@ -654,7 +697,7 @@ def complete_multipartite_graph(*subset_sizes):
     subset_sizes : tuple of integers or tuple of node iterables
        The arguments can either all be integer number of nodes or they
        can all be iterables of nodes. If integers, they represent the
-       number of vertices in each subset of the multipartite graph.
+       number of nodes in each subset of the multipartite graph.
        If iterables, each is used to create the nodes for that subset.
        The length of subset_sizes is the number of subsets.
 
@@ -669,11 +712,10 @@ def complete_multipartite_graph(*subset_sizes):
     Examples
     --------
     Creating a complete tripartite graph, with subsets of one, two, and three
-    vertices, respectively.
+    nodes, respectively.
 
-        >>> import networkx as nx
         >>> G = nx.complete_multipartite_graph(1, 2, 3)
-        >>> [G.nodes[u]['subset'] for u in G]
+        >>> [G.nodes[u]["subset"] for u in G]
         [0, 1, 1, 2, 2, 2]
         >>> list(G.edges(0))
         [(0, 1), (0, 2), (0, 3), (0, 4), (0, 5)]
@@ -682,13 +724,13 @@ def complete_multipartite_graph(*subset_sizes):
         >>> list(G.edges(4))
         [(4, 0), (4, 1), (4, 2)]
 
-        >>> G = nx.complete_multipartite_graph('a', 'bc', 'def')
-        >>> [G.nodes[u]['subset'] for u in sorted(G)]
+        >>> G = nx.complete_multipartite_graph("a", "bc", "def")
+        >>> [G.nodes[u]["subset"] for u in sorted(G)]
         [0, 1, 1, 2, 2, 2]
 
     Notes
     -----
-    This function generalizes several other graph generator functions.
+    This function generalizes several other graph builder functions.
 
     - If no subset sizes are given, this returns the null graph.
     - If a single subset size `n` is given, this returns the empty graph on
@@ -710,7 +752,7 @@ def complete_multipartite_graph(*subset_sizes):
 
     # set up subsets of nodes
     try:
-        extents = pairwise(accumulate((0,) + subset_sizes))
+        extents = pairwise(itertools.accumulate((0,) + subset_sizes))
         subsets = [range(start, end) for start, end in extents]
     except TypeError:
         subsets = subset_sizes
@@ -720,10 +762,10 @@ def complete_multipartite_graph(*subset_sizes):
     try:
         for (i, subset) in enumerate(subsets):
             G.add_nodes_from(subset, subset=i)
-    except TypeError:
-        raise NetworkXError("Arguments must be all ints or all iterables")
+    except TypeError as e:
+        raise NetworkXError("Arguments must be all ints or all iterables") from e
 
-    # Across subsets, all vertices should be adjacent.
+    # Across subsets, all nodes should be adjacent.
     # We can use itertools.combinations() because undirected.
     for subset1, subset2 in itertools.combinations(subsets, 2):
         G.add_edges_from(itertools.product(subset1, subset2))
