@@ -1,39 +1,22 @@
-# -*- coding: utf-8 -*-
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (hagberg@lanl.gov)
-#          Dan Schult (dschult@colgate.edu)
-#          Ben Edwards (BJEdwards@gmail.com)
-#          Arya McCarthy (admccarthy@smu.edu)
-#          Cole MacLean (maclean.cole@gmail.com)
-
 """Generators for geometric graphs.
 """
-from __future__ import division
 
 from bisect import bisect_left
-from itertools import combinations
-from itertools import product
+from itertools import accumulate, combinations, product
 from math import sqrt
 import math
-try:
-    from scipy.spatial import cKDTree as KDTree
-except ImportError:
-    _is_scipy_available = False
-else:
-    _is_scipy_available = True
 
 import networkx as nx
 from networkx.utils import nodes_or_number, py_random_state
 
-__all__ = ['geographical_threshold_graph', 'waxman_graph',
-           'navigable_small_world_graph', 'random_geometric_graph',
-           'soft_random_geometric_graph', 'thresholded_random_geometric_graph']
+__all__ = [
+    "geographical_threshold_graph",
+    "waxman_graph",
+    "navigable_small_world_graph",
+    "random_geometric_graph",
+    "soft_random_geometric_graph",
+    "thresholded_random_geometric_graph",
+]
 
 
 def euclidean(x, y):
@@ -46,31 +29,30 @@ def euclidean(x, y):
     return sqrt(sum((a - b) ** 2 for a, b in zip(x, y)))
 
 
-def _fast_edges(G, radius, p):
+def geometric_edges(G, radius, p):
     """Returns edge list of node pairs within `radius` of each other
-       using scipy KDTree and Minkowski distance metric `p`
 
-    Requires scipy to be installed.
+    Radius uses Minkowski distance metric `p`.
+    If scipy available, use scipy cKDTree to speed computation.
     """
-    pos = nx.get_node_attributes(G, 'pos')
-    nodes, coords = list(zip(*pos.items()))
-    kdtree = KDTree(coords)  # Cannot provide generator.
+    nodes_pos = G.nodes(data="pos")
+    try:
+        import scipy as sp
+        import scipy.spatial  # call as sp.spatial
+    except ImportError:
+        # no scipy KDTree so compute by for-loop
+        radius_p = radius ** p
+        edges = [
+            (u, v)
+            for (u, pu), (v, pv) in combinations(nodes_pos, 2)
+            if sum(abs(a - b) ** p for a, b in zip(pu, pv)) <= radius_p
+        ]
+        return edges
+    # scipy KDTree is available
+    nodes, coords = list(zip(*nodes_pos))
+    kdtree = sp.spatial.cKDTree(coords)  # Cannot provide generator.
     edge_indexes = kdtree.query_pairs(radius, p)
-    edges = ((nodes[u], nodes[v]) for u, v in edge_indexes)
-    return edges
-
-
-def _slow_edges(G, radius, p):
-    """Returns edge list of node pairs within `radius` of each other
-       using Minkowski distance metric `p`
-
-    Works without scipy, but in `O(n^2)` time.
-    """
-    # TODO This can be parallelized.
-    edges = []
-    for (u, pu), (v, pv) in combinations(G.nodes(data='pos'), 2):
-        if sum(abs(a - b) ** p for a, b in zip(pu, pv)) <= radius ** p:
-            edges.append((u, v))
+    edges = [(nodes[u], nodes[v]) for u, v in sorted(edge_indexes)]
     return edges
 
 
@@ -160,21 +142,17 @@ def random_geometric_graph(n, radius, dim=2, pos=None, p=2, seed=None):
     # Euclidean space of the specified dimension.
     if pos is None:
         pos = {v: [seed.random() for i in range(dim)] for v in nodes}
-    nx.set_node_attributes(G, pos, 'pos')
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-    else:
-        edges = _slow_edges(G, radius, p)
-    G.add_edges_from(edges)
+    nx.set_node_attributes(G, pos, "pos")
 
+    G.add_edges_from(geometric_edges(G, radius, p))
     return G
 
 
 @py_random_state(6)
 @nodes_or_number(0)
-def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
-                                seed=None):
+def soft_random_geometric_graph(
+    n, radius, dim=2, pos=None, p=2, p_dist=None, seed=None
+):
     r"""Returns a soft random geometric graph in the unit cube.
 
     The soft random geometric graph [1] model places `n` nodes uniformly at
@@ -262,26 +240,26 @@ def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
     >>> import math
     >>> n = 100
     >>> pos = {i: (random.gauss(0, 2), random.gauss(0, 2)) for i in range(n)}
-    >>> p_dist = lambda dist : math.exp(-dist)
+    >>> p_dist = lambda dist: math.exp(-dist)
     >>> G = nx.soft_random_geometric_graph(n, 0.2, pos=pos, p_dist=p_dist)
 
     References
     ----------
     .. [1] Penrose, Mathew D. "Connectivity of soft random geometric graphs."
            The Annals of Applied Probability 26.2 (2016): 986-1028.
-       [2] scipy.stats -
+    .. [2] scipy.stats -
            https://docs.scipy.org/doc/scipy/reference/tutorial/stats.html
 
     """
     n_name, nodes = n
     G = nx.Graph()
-    G.name = 'soft_random_geometric_graph({}, {}, {})'.format(n, radius, dim)
+    G.name = f"soft_random_geometric_graph({n}, {radius}, {dim})"
     G.add_nodes_from(nodes)
     # If no positions are provided, choose uniformly random vectors in
     # Euclidean space of the specified dimension.
     if pos is None:
         pos = {v: [seed.random() for i in range(dim)] for v in nodes}
-    nx.set_node_attributes(G, pos, 'pos')
+    nx.set_node_attributes(G, pos, "pos")
 
     # if p_dist function not supplied the default function is an exponential
     # distribution with rate parameter :math:`\lambda=1`.
@@ -290,32 +268,20 @@ def soft_random_geometric_graph(n, radius, dim=2, pos=None, p=2, p_dist=None,
         def p_dist(dist):
             return math.exp(-dist)
 
-    def should_join(pair):
-        u, v = pair
-        u_pos, v_pos = pos[u], pos[v]
-        dist = (sum(abs(a - b) ** p for a, b in zip(u_pos, v_pos)))**(1 / p)
-        # Check if dist <= radius parameter. This check is redundant if scipy
-        # is available and _fast_edges routine is used, but provides the
-        # check in case scipy is not available and all edge combinations
-        # need to be checked
-        if dist <= radius:
-            return seed.random() < p_dist(dist)
-        else:
-            return False
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-        G.add_edges_from(filter(should_join, edges))
-    else:
-        G.add_edges_from(filter(should_join, combinations(G, 2)))
+    def should_join(edge):
+        u, v = edge
+        dist = (sum(abs(a - b) ** p for a, b in zip(pos[u], pos[v]))) ** (1 / p)
+        return seed.random() < p_dist(dist)
 
+    G.add_edges_from(filter(should_join, geometric_edges(G, radius, p)))
     return G
 
 
 @py_random_state(7)
 @nodes_or_number(0)
-def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
-                                 metric=None, p_dist=None, seed=None):
+def geographical_threshold_graph(
+    n, theta, dim=2, pos=None, weight=None, metric=None, p_dist=None, seed=None
+):
     r"""Returns a geographical threshold graph.
 
     The geographical threshold graph model places $n$ nodes uniformly at
@@ -415,16 +381,6 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
     If node positions are not specified they are randomly assigned from the
     uniform distribution.
 
-    Starting in NetworkX 2.1 the parameter ``alpha`` is deprecated and replaced
-    with the customizable ``p_dist`` function parameter, which defaults to r^-2
-    if ``p_dist`` is not supplied. To reproduce networks of earlier NetworkX
-    versions, a custom function needs to be defined and passed as the
-    ``p_dist`` parameter. For example, if the parameter ``alpha`` = 2 was used
-    in NetworkX 2.0, the custom function def custom_dist(r): r**-2 can be
-    passed in versions >=2.1 as the parameter p_dist = custom_dist to
-    produce an equivalent network. Note the change in sign from +2 to -2 in
-    this parameter change.
-
     References
     ----------
     .. [1] Masuda, N., Miwa, H., Konno, N.:
@@ -450,13 +406,14 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
     # If no distance metric is provided, use Euclidean distance.
     if metric is None:
         metric = euclidean
-    nx.set_node_attributes(G, weight, 'weight')
-    nx.set_node_attributes(G, pos, 'pos')
+    nx.set_node_attributes(G, weight, "weight")
+    nx.set_node_attributes(G, pos, "pos")
 
     # if p_dist is not supplied, use default r^-2
     if p_dist is None:
+
         def p_dist(r):
-            return r**-2
+            return r ** -2
 
     # Returns ``True`` if and only if the nodes whose attributes are
     # ``du`` and ``dv`` should be joined, according to the threshold
@@ -473,8 +430,9 @@ def geographical_threshold_graph(n, theta, dim=2, pos=None, weight=None,
 
 @py_random_state(6)
 @nodes_or_number(0)
-def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
-                 metric=None, seed=None):
+def waxman_graph(
+    n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1), metric=None, seed=None
+):
     r"""Returns a Waxman random graph.
 
     The Waxman random graph model places `n` nodes uniformly at random
@@ -565,7 +523,7 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
     (xmin, ymin, xmax, ymax) = domain
     # Each node gets a uniformly random position in the given rectangle.
     pos = {v: (seed.uniform(xmin, xmax), seed.uniform(ymin, ymax)) for v in G}
-    nx.set_node_attributes(G, pos, 'pos')
+    nx.set_node_attributes(G, pos, "pos")
     # If no distance metric is provided, use Euclidean distance.
     if metric is None:
         metric = euclidean
@@ -578,9 +536,13 @@ def waxman_graph(n, beta=0.4, alpha=0.1, L=None, domain=(0, 0, 1, 1),
     if L is None:
         L = max(metric(x, y) for x, y in combinations(pos.values(), 2))
 
-        def dist(u, v): return metric(pos[u], pos[v])
+        def dist(u, v):
+            return metric(pos[u], pos[v])
+
     else:
-        def dist(u, v): return seed.random() * L
+
+        def dist(u, v):
+            return seed.random() * L
 
     # `pair` is the pair of nodes to decide whether to join.
     def should_join(pair):
@@ -637,11 +599,11 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
     .. [1] J. Kleinberg. The small-world phenomenon: An algorithmic
        perspective. Proc. 32nd ACM Symposium on Theory of Computing, 2000.
     """
-    if (p < 1):
+    if p < 1:
         raise nx.NetworkXException("p must be >= 1")
-    if (q < 0):
+    if q < 0:
         raise nx.NetworkXException("q must be >= 0")
-    if (r < 0):
+    if r < 0:
         raise nx.NetworkXException("r must be >= 1")
 
     G = nx.DiGraph()
@@ -654,8 +616,8 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
             d = sum((abs(b - a) for a, b in zip(p1, p2)))
             if d <= p:
                 G.add_edge(p1, p2)
-            probs.append(d**-r)
-        cdf = list(nx.utils.accumulate(probs))
+            probs.append(d ** -r)
+        cdf = list(accumulate(probs))
         for _ in range(q):
             target = nodes[bisect_left(cdf, seed.uniform(0, cdf[-1]))]
             G.add_edge(p1, target)
@@ -664,8 +626,9 @@ def navigable_small_world_graph(n, p=1, q=1, r=2, dim=2, seed=None):
 
 @py_random_state(7)
 @nodes_or_number(0)
-def thresholded_random_geometric_graph(n, radius, theta, dim=2,
-                                       pos=None, weight=None, p=2, seed=None):
+def thresholded_random_geometric_graph(
+    n, radius, theta, dim=2, pos=None, weight=None, p=2, seed=None
+):
     r"""Returns a thresholded random geometric graph in the unit cube.
 
     The thresholded random geometric graph [1] model places `n` nodes
@@ -694,7 +657,7 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
         A dictionary keyed by node with node positions as values.
     weight : dict, optional
         Node weights as a dictionary of numbers keyed by node.
-    p : float, optional
+    p : float, optional (default 2)
         Which Minkowski distance metric to use.  `p` has to meet the condition
         ``1 <= p <= infinity``.
 
@@ -765,8 +728,7 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
 
     n_name, nodes = n
     G = nx.Graph()
-    namestr = 'thresholded_random_geometric_graph({}, {}, {}, {})'
-    G.name = namestr.format(n, radius, theta, dim)
+    G.name = f"thresholded_random_geometric_graph({n}, {radius}, {theta}, {dim})"
     G.add_nodes_from(nodes)
     # If no weights are provided, choose them from an exponential
     # distribution.
@@ -777,32 +739,13 @@ def thresholded_random_geometric_graph(n, radius, theta, dim=2,
     if pos is None:
         pos = {v: [seed.random() for i in range(dim)] for v in nodes}
     # If no distance metric is provided, use Euclidean distance.
+    nx.set_node_attributes(G, weight, "weight")
+    nx.set_node_attributes(G, pos, "pos")
 
-    nx.set_node_attributes(G, weight, 'weight')
-    nx.set_node_attributes(G, pos, 'pos')
-
-    # Returns ``True`` if and only if the nodes whose attributes are
-    # ``du`` and ``dv`` should be joined, according to the threshold
-    # condition and node pairs are within the maximum connection
-    # distance, ``radius``.
-    def should_join(pair):
-        u, v = pair
-        u_weight, v_weight = weight[u], weight[v]
-        u_pos, v_pos = pos[u], pos[v]
-        dist = (sum(abs(a - b) ** p for a, b in zip(u_pos, v_pos)))**(1 / p)
-        # Check if dist is <= radius parameter. This check is redundant if
-        # scipy is available and _fast_edges routine is used, but provides
-        # the check in case scipy is not available and all edge combinations
-        # need to be checked
-        if dist <= radius:
-            return theta <= u_weight + v_weight
-        else:
-            return False
-
-    if _is_scipy_available:
-        edges = _fast_edges(G, radius, p)
-        G.add_edges_from(filter(should_join, edges))
-    else:
-        G.add_edges_from(filter(should_join, combinations(G, 2)))
-
+    edges = (
+        (u, v)
+        for u, v in geometric_edges(G, radius, p)
+        if weight[u] + weight[v] >= theta
+    )
+    G.add_edges_from(edges)
     return G
