@@ -1,17 +1,3 @@
-# -*- coding: utf-8 -*-
-#    Copyright (C) 2006-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors:
-#    Aric Hagberg <aric.hagberg@gmail.com>
-#    Dan Schult <dschult@colgate.edu>
-#    Ben Edwards <bedwards@cs.unm.edu>
-#    Neil Girdhar <neil.girdhar@mcgill.ca>
-#
 """Algorithms for directed acyclic graphs (DAGs).
 
 Note that most of these functions are only guaranteed to work for DAGs.
@@ -19,35 +5,32 @@ In general, these functions do not check for acyclic-ness, so it is up
 to the user to check for that.
 """
 
-from collections import defaultdict, deque
-from fractions import gcd
+from collections import deque
+from math import gcd
 from functools import partial
-from itertools import chain
-from itertools import product
-from itertools import starmap
+from itertools import chain, product, starmap
 import heapq
 
 import networkx as nx
-from networkx.generators.trees import NIL
-from networkx.utils import arbitrary_element
-from networkx.utils import consume
-from networkx.utils import pairwise
-from networkx.utils import generate_unique_node
-from networkx.utils import not_implemented_for
+from networkx.utils import arbitrary_element, pairwise, not_implemented_for
 
-__all__ = ['descendants',
-           'ancestors',
-           'topological_sort',
-           'lexicographical_topological_sort',
-           'all_topological_sorts',
-           'is_directed_acyclic_graph',
-           'is_aperiodic',
-           'transitive_closure',
-           'transitive_reduction',
-           'antichains',
-           'dag_longest_path',
-           'dag_longest_path_length',
-           'dag_to_branching']
+__all__ = [
+    "descendants",
+    "ancestors",
+    "topological_sort",
+    "lexicographical_topological_sort",
+    "all_topological_sorts",
+    "topological_generations",
+    "is_directed_acyclic_graph",
+    "is_aperiodic",
+    "transitive_closure",
+    "transitive_closure_dag",
+    "transitive_reduction",
+    "antichains",
+    "dag_longest_path",
+    "dag_longest_path_length",
+    "dag_to_branching",
+]
 
 chaini = chain.from_iterable
 
@@ -58,7 +41,7 @@ def descendants(G, source):
     Parameters
     ----------
     G : NetworkX DiGraph
-        A directed acyclic graph (DAG)
+        A directed graph
     source : node in `G`
 
     Returns
@@ -67,8 +50,8 @@ def descendants(G, source):
         The descendants of `source` in `G`
     """
     if not G.has_node(source):
-        raise nx.NetworkXError("The node %s is not in the graph." % source)
-    des = set(n for n, d in nx.shortest_path_length(G, source=source).items())
+        raise nx.NetworkXError(f"The node {source} is not in the graph.")
+    des = {n for n, d in nx.shortest_path_length(G, source=source).items()}
     return des - {source}
 
 
@@ -78,7 +61,7 @@ def ancestors(G, source):
     Parameters
     ----------
     G : NetworkX DiGraph
-        A directed acyclic graph (DAG)
+        A directed graph
     source : node in `G`
 
     Returns
@@ -87,15 +70,16 @@ def ancestors(G, source):
         The ancestors of source in G
     """
     if not G.has_node(source):
-        raise nx.NetworkXError("The node %s is not in the graph." % source)
-    anc = set(n for n, d in nx.shortest_path_length(G, target=source).items())
+        raise nx.NetworkXError(f"The node {source} is not in the graph.")
+    anc = {n for n, d in nx.shortest_path_length(G, target=source).items()}
     return anc - {source}
 
 
 def has_cycle(G):
     """Decides whether the directed graph has a cycle."""
     try:
-        consume(topological_sort(G))
+        # Feed the entire iterator into a zero-length deque.
+        deque(topological_sort(G), maxlen=0)
     except nx.NetworkXUnfeasible:
         return True
     else:
@@ -118,22 +102,100 @@ def is_directed_acyclic_graph(G):
     return G.is_directed() and not has_cycle(G)
 
 
-def topological_sort(G):
-    """Returns a generator of nodes in topologically sorted order.
+def topological_generations(G):
+    """Stratifies a DAG into generations.
 
-    A topological sort is a nonunique permutation of the nodes such that an
-    edge from u to v implies that u appears before v in the topological sort
-    order.
+    A topological generation is node collection in which ancestors of a node in each
+    generation are guaranteed to be in a previous generation, and any descendants of
+    a node are guaranteed to be in a following generation. Nodes are guaranteed to
+    be in the earliest possible generation that they can belong to.
 
     Parameters
     ----------
     G : NetworkX digraph
         A directed acyclic graph (DAG)
 
-    Returns
-    -------
-    iterable
-        An iterable of node names in topological sorted order.
+    Yields
+    ------
+    sets of nodes
+        Yields sets of nodes representing each generation.
+
+    Raises
+    ------
+    NetworkXError
+        Generations are defined for directed graphs only. If the graph
+        `G` is undirected, a :exc:`NetworkXError` is raised.
+
+    NetworkXUnfeasible
+        If `G` is not a directed acyclic graph (DAG) no topological generations
+        exist and a :exc:`NetworkXUnfeasible` exception is raised.  This can also
+        be raised if `G` is changed while the returned iterator is being processed
+
+    RuntimeError
+        If `G` is changed while the returned iterator is being processed.
+
+    Examples
+    --------
+    >>> DG = nx.DiGraph([(2, 1), (3, 1)])
+    >>> [sorted(generation) for generation in nx.topological_generations(DG)]
+    [[2, 3], [1]]
+
+    Notes
+    -----
+    The generation in which a node resides can also be determined by taking the
+    max-path-distance from the node to the farthest leaf node. That value can
+    be obtained with this function using `enumerate(topological_generations(G))`.
+
+    See also
+    --------
+    topological_sort
+    """
+    if not G.is_directed():
+        raise nx.NetworkXError("Topological sort not defined on undirected graphs.")
+
+    multigraph = G.is_multigraph()
+    indegree_map = {v: d for v, d in G.in_degree() if d > 0}
+    zero_indegree = [v for v, d in G.in_degree() if d == 0]
+
+    while zero_indegree:
+        this_generation = zero_indegree
+        zero_indegree = []
+        for node in this_generation:
+            if node not in G:
+                raise RuntimeError("Graph changed during iteration")
+            for child in G.neighbors(node):
+                try:
+                    indegree_map[child] -= len(G[node][child]) if multigraph else 1
+                except KeyError as e:
+                    raise RuntimeError("Graph changed during iteration") from e
+                if indegree_map[child] == 0:
+                    zero_indegree.append(child)
+                    del indegree_map[child]
+        yield this_generation
+
+    if indegree_map:
+        raise nx.NetworkXUnfeasible(
+            "Graph contains a cycle or graph changed during iteration"
+        )
+
+
+def topological_sort(G):
+    """Returns a generator of nodes in topologically sorted order.
+
+    A topological sort is a nonunique permutation of the nodes of a
+    directed graph such that an edge from u to v implies that u
+    appears before v in the topological sort order. This ordering is
+    valid only if the graph has no directed cycles.
+
+    Parameters
+    ----------
+    G : NetworkX digraph
+        A directed acyclic graph (DAG)
+
+    Yields
+    ------
+    nodes
+        Yields the nodes in topological sorted order.
 
     Raises
     ------
@@ -181,32 +243,8 @@ def topological_sort(G):
     .. [1] Manber, U. (1989).
        *Introduction to Algorithms - A Creative Approach.* Addison-Wesley.
     """
-    if not G.is_directed():
-        raise nx.NetworkXError(
-            "Topological sort not defined on undirected graphs.")
-
-    indegree_map = {v: d for v, d in G.in_degree() if d > 0}
-    # These nodes have zero indegree and ready to be returned.
-    zero_indegree = [v for v, d in G.in_degree() if d == 0]
-
-    while zero_indegree:
-        node = zero_indegree.pop()
-        if node not in G:
-            raise RuntimeError("Graph changed during iteration")
-        for _, child in G.edges(node):
-            try:
-                indegree_map[child] -= 1
-            except KeyError:
-                raise RuntimeError("Graph changed during iteration")
-            if indegree_map[child] == 0:
-                zero_indegree.append(child)
-                del indegree_map[child]
-
-        yield node
-
-    if indegree_map:
-        raise nx.NetworkXUnfeasible("Graph contains a cycle or graph changed "
-                                    "during iteration")
+    for generation in nx.topological_generations(G):
+        yield from generation
 
 
 def lexicographical_topological_sort(G, key=None):
@@ -260,14 +298,18 @@ def lexicographical_topological_sort(G, key=None):
        *Introduction to Algorithms - A Creative Approach.* Addison-Wesley.
     """
     if not G.is_directed():
-        raise nx.NetworkXError(
-            "Topological sort not defined on undirected graphs.")
+        msg = "Topological sort not defined on undirected graphs."
+        raise nx.NetworkXError(msg)
 
     if key is None:
-        def key(x): return x
+
+        def key(node):
+            return node
+
+    nodeid_map = {n: i for i, n in enumerate(G)}
 
     def create_tuple(node):
-        return key(node), node
+        return key(node), nodeid_map[node], node
 
     indegree_map = {v: d for v, d in G.in_degree() if d > 0}
     # These nodes have zero indegree and ready to be returned.
@@ -275,15 +317,15 @@ def lexicographical_topological_sort(G, key=None):
     heapq.heapify(zero_indegree)
 
     while zero_indegree:
-        _, node = heapq.heappop(zero_indegree)
+        _, _, node = heapq.heappop(zero_indegree)
 
         if node not in G:
             raise RuntimeError("Graph changed during iteration")
         for _, child in G.edges(node):
             try:
                 indegree_map[child] -= 1
-            except KeyError:
-                raise RuntimeError("Graph changed during iteration")
+            except KeyError as e:
+                raise RuntimeError("Graph changed during iteration") from e
             if indegree_map[child] == 0:
                 heapq.heappush(zero_indegree, create_tuple(child))
                 del indegree_map[child]
@@ -291,11 +333,11 @@ def lexicographical_topological_sort(G, key=None):
         yield node
 
     if indegree_map:
-        raise nx.NetworkXUnfeasible("Graph contains a cycle or graph changed "
-                                    "during iteration")
+        msg = "Graph contains a cycle or graph changed during iteration"
+        raise nx.NetworkXUnfeasible(msg)
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def all_topological_sorts(G):
     """Returns a generator of _all_ topological sorts of the directed graph G.
 
@@ -342,8 +384,7 @@ def all_topological_sorts(G):
        Elsevier (North-Holland), Amsterdam
     """
     if not G.is_directed():
-        raise nx.NetworkXError(
-            "Topological sort not defined on undirected graphs.")
+        raise nx.NetworkXError("Topological sort not defined on undirected graphs.")
 
     # the names of count and D are chosen to match the global variables in [1]
     # number of edges originating in a vertex v
@@ -450,14 +491,13 @@ def is_aperiodic(G):
        A Multidisciplinary Approach, CRC Press.
     """
     if not G.is_directed():
-        raise nx.NetworkXError(
-            "is_aperiodic not defined for undirected graphs")
+        raise nx.NetworkXError("is_aperiodic not defined for undirected graphs")
 
     s = arbitrary_element(G)
     levels = {s: 0}
     this_level = [s]
     g = 0
-    l = 1
+    lev = 1
     while this_level:
         next_level = []
         for u in this_level:
@@ -466,27 +506,39 @@ def is_aperiodic(G):
                     g = gcd(g, levels[u] - levels[v] + 1)
                 else:  # Tree Edge
                     next_level.append(v)
-                    levels[v] = l
+                    levels[v] = lev
         this_level = next_level
-        l += 1
+        lev += 1
     if len(levels) == len(G):  # All nodes in tree
         return g == 1
     else:
         return g == 1 and nx.is_aperiodic(G.subgraph(set(G) - set(levels)))
 
 
-@not_implemented_for('undirected')
-def transitive_closure(G):
-    """ Returns transitive closure of a directed graph
+@not_implemented_for("undirected")
+def transitive_closure(G, reflexive=False):
+    """Returns transitive closure of a directed graph
 
     The transitive closure of G = (V,E) is a graph G+ = (V,E+) such that
-    for all v,w in V there is an edge (v,w) in E+ if and only if there
-    is a non-null path from v to w in G.
+    for all v, w in V there is an edge (v, w) in E+ if and only if there
+    is a path from v to w in G.
+
+    Handling of paths from v to v has some flexibility within this definition.
+    A reflexive transitive closure creates a self-loop for the path
+    from v to v of length 0. The usual transitive closure creates a
+    self-loop only if a cycle exists (a path from v to v with length > 0).
+    We also allow an option for no self-loops.
 
     Parameters
     ----------
     G : NetworkX DiGraph
         A directed graph
+    reflexive : Bool or None, optional (default: False)
+        Determines when cycles create self-loops in the Transitive Closure.
+        If True, trivial cycles (length 0) create self-loops. The result
+        is a reflexive tranistive closure of G.
+        If False (the default) non-trivial cycles create self-loops.
+        If None, self-loops are not created.
 
     Returns
     -------
@@ -502,17 +554,81 @@ def transitive_closure(G):
     ----------
     .. [1] http://www.ics.uci.edu/~eppstein/PADS/PartialOrder.py
 
+    TODO this function applies to all directed graphs and is probably misplaced
+         here in dag.py
     """
+    if reflexive is None:
+        TC = G.copy()
+        for v in G:
+            edges = ((v, u) for u in nx.dfs_preorder_nodes(G, v) if v != u)
+            TC.add_edges_from(edges)
+        return TC
+    if reflexive is True:
+        TC = G.copy()
+        for v in G:
+            edges = ((v, u) for u in nx.dfs_preorder_nodes(G, v))
+            TC.add_edges_from(edges)
+        return TC
+    # reflexive is False
     TC = G.copy()
     for v in G:
-        TC.add_edges_from((v, u) for u in nx.dfs_preorder_nodes(G, source=v)
-                          if v != u)
+        edges = ((v, w) for u, w in nx.edge_dfs(G, v))
+        TC.add_edges_from(edges)
     return TC
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
+def transitive_closure_dag(G, topo_order=None):
+    """Returns the transitive closure of a directed acyclic graph.
+
+    This function is faster than the function `transitive_closure`, but fails
+    if the graph has a cycle.
+
+    The transitive closure of G = (V,E) is a graph G+ = (V,E+) such that
+    for all v, w in V there is an edge (v, w) in E+ if and only if there
+    is a non-null path from v to w in G.
+
+    Parameters
+    ----------
+    G : NetworkX DiGraph
+        A directed acyclic graph (DAG)
+
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
+    Returns
+    -------
+    NetworkX DiGraph
+        The transitive closure of `G`
+
+    Raises
+    ------
+    NetworkXNotImplemented
+        If `G` is not directed
+    NetworkXUnfeasible
+        If `G` has a cycle
+
+    Notes
+    -----
+    This algorithm is probably simple enough to be well-known but I didn't find
+    a mention in the literature.
+    """
+    if topo_order is None:
+        topo_order = list(topological_sort(G))
+
+    TC = G.copy()
+
+    # idea: traverse vertices following a reverse topological order, connecting
+    # each vertex to its descendants at distance 2 as we go
+    for v in reversed(topo_order):
+        TC.add_edges_from((v, u) for u in nx.descendants_at_distance(TC, v, 2))
+
+    return TC
+
+
+@not_implemented_for("undirected")
 def transitive_reduction(G):
-    """ Returns transitive reduction of a directed graph
+    """Returns transitive reduction of a directed graph
 
     The transitive reduction of G = (V,E) is a graph G- = (V,E-) such that
     for all v,w in V there is an edge (v,w) in E- if and only if (v,w) is
@@ -534,6 +650,27 @@ def transitive_reduction(G):
         If `G` is not a directed acyclic graph (DAG) transitive reduction is
         not uniquely defined and a :exc:`NetworkXError` exception is raised.
 
+    Examples
+    --------
+    To perform transitive reduction on a DiGraph:
+
+    >>> DG = nx.DiGraph([(1, 2), (2, 3), (1, 3)])
+    >>> TR = nx.transitive_reduction(DG)
+    >>> list(TR.edges)
+    [(1, 2), (2, 3)]
+
+    To avoid unnecessary data copies, this implementation does not return a
+    DiGraph with node/edge data.
+    To perform transitive reduction on a DiGraph and transfer node/edge data:
+
+    >>> DG = nx.DiGraph()
+    >>> DG.add_edges_from([(1, 2), (2, 3), (1, 3)], color='red')
+    >>> TR = nx.transitive_reduction(DG)
+    >>> TR.add_nodes_from(DG.nodes(data=True))
+    >>> TR.add_edges_from((u, v, DG.edges[u, v]) for u, v in TR.edges)
+    >>> list(TR.edges(data=True))
+    [(1, 2, {'color': 'red'}), (2, 3, {'color': 'red'})]
+
     References
     ----------
     https://en.wikipedia.org/wiki/Transitive_reduction
@@ -561,8 +698,8 @@ def transitive_reduction(G):
     return TR
 
 
-@not_implemented_for('undirected')
-def antichains(G):
+@not_implemented_for("undirected")
+def antichains(G, topo_order=None):
     """Generates antichains from a directed acyclic graph (DAG).
 
     An antichain is a subset of a partially ordered set such that any
@@ -573,6 +710,9 @@ def antichains(G):
     G : NetworkX DiGraph
         A directed acyclic graph (DAG)
 
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
     Returns
     -------
     generator object
@@ -598,8 +738,12 @@ def antichains(G):
     .. [1] Free Lattices, by R. Freese, J. Jezek and J. B. Nation,
        AMS, Vol 42, 1995, p. 226.
     """
-    TC = nx.transitive_closure(G)
-    antichains_stacks = [([], list(reversed(list(nx.topological_sort(G)))))]
+    if topo_order is None:
+        topo_order = list(nx.topological_sort(G))
+
+    TC = nx.transitive_closure_dag(G, topo_order)
+    antichains_stacks = [([], list(reversed(topo_order)))]
+
     while antichains_stacks:
         (antichain, stack) = antichains_stacks.pop()
         # Invariant:
@@ -609,13 +753,12 @@ def antichains(G):
         while stack:
             x = stack.pop()
             new_antichain = antichain + [x]
-            new_stack = [
-                t for t in stack if not ((t in TC[x]) or (x in TC[t]))]
+            new_stack = [t for t in stack if not ((t in TC[x]) or (x in TC[t]))]
             antichains_stacks.append((new_antichain, new_stack))
 
 
-@not_implemented_for('undirected')
-def dag_longest_path(G, weight='weight', default_weight=1):
+@not_implemented_for("undirected")
+def dag_longest_path(G, weight="weight", default_weight=1, topo_order=None):
     """Returns the longest path in a directed acyclic graph (DAG).
 
     If `G` has edges with `weight` attribute the edge data are used as
@@ -632,6 +775,9 @@ def dag_longest_path(G, weight='weight', default_weight=1):
     default_weight : int, optional
         The weight of edges that do not have a weight attribute
 
+    topo_order: list or tuple, optional
+        A topological order for G (if None, the function will compute one)
+
     Returns
     -------
     list
@@ -649,14 +795,22 @@ def dag_longest_path(G, weight='weight', default_weight=1):
     """
     if not G:
         return []
+
+    if topo_order is None:
+        topo_order = nx.topological_sort(G)
+
     dist = {}  # stores {v : (length, u)}
-    for v in nx.topological_sort(G):
-        us = [(dist[u][0] + data.get(weight, default_weight), u)
-              for u, data in G.pred[v].items()]
+    for v in topo_order:
+        us = [
+            (dist[u][0] + data.get(weight, default_weight), u)
+            for u, data in G.pred[v].items()
+        ]
+
         # Use the best predecessor if there is one and its distance is
         # non-negative, otherwise terminate.
         maxu = max(us, key=lambda x: x[0]) if us else (0, v)
         dist[v] = maxu if maxu[0] >= 0 else (0, v)
+
     u = None
     v = max(dist, key=lambda x: dist[x][0])
     path = []
@@ -664,12 +818,13 @@ def dag_longest_path(G, weight='weight', default_weight=1):
         path.append(v)
         u = v
         v = dist[v][1]
+
     path.reverse()
     return path
 
 
-@not_implemented_for('undirected')
-def dag_longest_path_length(G, weight='weight', default_weight=1):
+@not_implemented_for("undirected")
+def dag_longest_path_length(G, weight="weight", default_weight=1):
     """Returns the longest path length in a DAG
 
     Parameters
@@ -723,8 +878,8 @@ def root_to_leaf_paths(G):
     return chaini(starmap(all_paths, product(roots, leaves)))
 
 
-@not_implemented_for('multigraph')
-@not_implemented_for('undirected')
+@not_implemented_for("multigraph")
+@not_implemented_for("undirected")
 def dag_to_branching(G):
     """Returns a branching representing all (overlapping) paths from
     root nodes to leaf nodes in the given directed acyclic graph.
@@ -777,16 +932,16 @@ def dag_to_branching(G):
         >>> from collections import defaultdict
         >>> from operator import itemgetter
         >>>
-        >>> G = nx.DiGraph(nx.utils.pairwise('abd'))
-        >>> G.add_edges_from(nx.utils.pairwise('acd'))
+        >>> G = nx.DiGraph(nx.utils.pairwise("abd"))
+        >>> G.add_edges_from(nx.utils.pairwise("acd"))
         >>> B = nx.dag_to_branching(G)
         >>>
         >>> sources = defaultdict(set)
-        >>> for v, source in B.nodes(data='source'):
+        >>> for v, source in B.nodes(data="source"):
         ...     sources[source].add(v)
-        >>> len(sources['a'])
+        >>> len(sources["a"])
         1
-        >>> len(sources['d'])
+        >>> len(sources["d"])
         2
 
     To copy node attributes from the original graph to the new graph,
@@ -795,7 +950,7 @@ def dag_to_branching(G):
 
         >>> for source, nodes in sources.items():
         ...     for v in nodes:
-        ...         B.node[v].update(G.node[source])
+        ...         B.nodes[v].update(G.nodes[source])
 
     Notes
     -----
@@ -811,11 +966,11 @@ def dag_to_branching(G):
 
     """
     if has_cycle(G):
-        msg = 'dag_to_branching is only defined for acyclic graphs'
+        msg = "dag_to_branching is only defined for acyclic graphs"
         raise nx.HasACycle(msg)
     paths = root_to_leaf_paths(G)
-    B, root = nx.prefix_tree(paths)
-    # Remove the synthetic `root` and `NIL` nodes in the prefix tree.
-    B.remove_node(root)
-    B.remove_node(NIL)
+    B = nx.prefix_tree(paths)
+    # Remove the synthetic `root`(0) and `NIL`(-1) nodes from the tree
+    B.remove_node(0)
+    B.remove_node(-1)
     return B
