@@ -1,40 +1,56 @@
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg <hagberg@lanl.gov>
-#          Pieter Swart <swart@lanl.gov>
-#          Dan Schult <dschult@colgate.edu>
 """Functional interface to graph methods and assorted utilities.
 """
-from __future__ import division
 
 from collections import Counter
 from itertools import chain
-try:
-    from itertools import zip_longest
-except ImportError:
-    from itertools import izip_longest as zip_longest
 
 import networkx as nx
 from networkx.utils import pairwise, not_implemented_for
+from networkx.classes.graphviews import subgraph_view, reverse_view
 
-__all__ = ['nodes', 'edges', 'degree', 'degree_histogram', 'neighbors',
-           'number_of_nodes', 'number_of_edges', 'density',
-           'is_directed', 'info', 'freeze', 'is_frozen', 'subgraph',
-           'induced_subgraph', 'edge_subgraph', 'restricted_view',
-           'reverse_view', 'to_directed', 'to_undirected',
-           'add_star', 'add_path', 'add_cycle',
-           'create_empty_copy', 'set_node_attributes',
-           'get_node_attributes', 'set_edge_attributes',
-           'get_edge_attributes', 'all_neighbors', 'non_neighbors',
-           'non_edges', 'common_neighbors', 'is_weighted',
-           'is_negatively_weighted', 'is_empty',
-           'selfloop_edges', 'nodes_with_selfloops', 'number_of_selfloops',
-           ]
+
+__all__ = [
+    "nodes",
+    "edges",
+    "degree",
+    "degree_histogram",
+    "neighbors",
+    "number_of_nodes",
+    "number_of_edges",
+    "density",
+    "is_directed",
+    "info",
+    "freeze",
+    "is_frozen",
+    "subgraph",
+    "subgraph_view",
+    "induced_subgraph",
+    "reverse_view",
+    "edge_subgraph",
+    "restricted_view",
+    "to_directed",
+    "to_undirected",
+    "add_star",
+    "add_path",
+    "add_cycle",
+    "create_empty_copy",
+    "set_node_attributes",
+    "get_node_attributes",
+    "set_edge_attributes",
+    "get_edge_attributes",
+    "all_neighbors",
+    "non_neighbors",
+    "non_edges",
+    "common_neighbors",
+    "is_weighted",
+    "is_negatively_weighted",
+    "is_empty",
+    "selfloop_edges",
+    "nodes_with_selfloops",
+    "number_of_selfloops",
+    "path_weight",
+    "is_path",
+]
 
 
 def nodes(G):
@@ -60,7 +76,7 @@ def degree(G, nbunch=None, weight=None):
 
 
 def neighbors(G, n):
-    """Returns a list of nodes connected to node n. """
+    """Returns a list of nodes connected to node n."""
     return G.neighbors(n)
 
 
@@ -70,7 +86,7 @@ def number_of_nodes(G):
 
 
 def number_of_edges(G):
-    """Returns the number of edges in the graph. """
+    """Returns the number of edges in the graph."""
     return G.number_of_edges()
 
 
@@ -133,7 +149,7 @@ def degree_histogram(G):
 
 
 def is_directed(G):
-    """ Return True if graph is directed."""
+    """Return True if graph is directed."""
     return G.is_directed()
 
 
@@ -158,9 +174,9 @@ def freeze(G):
     >>> G = nx.path_graph(4)
     >>> G = nx.freeze(G)
     >>> try:
-    ...    G.add_edge(4, 5)
+    ...     G.add_edge(4, 5)
     ... except nx.NetworkXError as e:
-    ...    print(str(e))
+    ...     print(str(e))
     Frozen graph can't be modified
 
     Notes
@@ -295,7 +311,7 @@ def add_cycle(G_to_add_to, nodes_for_cycle, **attr):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> nx.add_cycle(G, [0, 1, 2, 3])
     >>> nx.add_cycle(G, [10, 11, 12], weight=7)
     """
@@ -305,7 +321,9 @@ def add_cycle(G_to_add_to, nodes_for_cycle, **attr):
     except StopIteration:
         return
     G_to_add_to.add_node(first_node)
-    G_to_add_to.add_edges_from(pairwise(chain((first_node,), nlist), cyclic=True), **attr)
+    G_to_add_to.add_edges_from(
+        pairwise(chain((first_node,), nlist), cyclic=True), **attr
+    )
 
 
 def subgraph(G, nbunch):
@@ -366,7 +384,6 @@ def induced_subgraph(G, nbunch):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(4)  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> H = G.subgraph([0, 1, 2])
     >>> list(H.edges)
@@ -409,7 +426,6 @@ def edge_subgraph(G, edges):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(5)
     >>> H = G.edge_subgraph([(0, 1), (3, 4)])
     >>> list(H.nodes)
@@ -470,7 +486,6 @@ def restricted_view(G, nodes, edges):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.path_graph(5)
     >>> H = nx.restricted_view(G, [0], [(1, 2), (3, 4)])
     >>> list(H.nodes)
@@ -493,15 +508,6 @@ def restricted_view(G, nodes, edges):
     return nx.graphviews.subgraph_view(G, hide_nodes, hide_edges)
 
 
-@not_implemented_for('undirected')
-def reverse_view(digraph):
-    """Provide a reverse view of the digraph with edges reversed.
-
-    Identical to digraph.reverse(copy=False)
-    """
-    return nx.graphviews.reverse_view(digraph)
-
-
 def to_directed(graph):
     """Returns a directed view of the graph `graph`.
 
@@ -509,7 +515,7 @@ def to_directed(graph):
     Note that graph.to_directed defaults to `as_view=False`
     while this function always provides a view.
     """
-    return graph.to_directed()
+    return graph.to_directed(as_view=True)
 
 
 def to_undirected(graph):
@@ -534,7 +540,7 @@ def create_empty_copy(G, with_data=True):
        Propagate Graph and Nodes data to the new graph.
 
     See Also
-    -----
+    --------
     empty_graph
 
     """
@@ -546,7 +552,10 @@ def create_empty_copy(G, with_data=True):
 
 
 def info(G, n=None):
-    """Print short summary of information for the graph G or the node n.
+    """Return a summary of information for the graph G or a single node n.
+
+    The summary includes the number of nodes and edges, or neighbours for a single
+    node.
 
     Parameters
     ----------
@@ -554,32 +563,27 @@ def info(G, n=None):
        A graph
     n : node (any hashable)
        A node in the graph G
-    """
-    info = ''  # append this all to a string
-    if n is None:
-        info += "Name: %s\n" % G.name
-        type_name = [type(G).__name__]
-        info += "Type: %s\n" % ",".join(type_name)
-        info += "Number of nodes: %d\n" % G.number_of_nodes()
-        info += "Number of edges: %d\n" % G.number_of_edges()
-        nnodes = G.number_of_nodes()
-        if len(G) > 0:
-            if G.is_directed():
-                deg = sum(d for n, d in G.in_degree()) / float(nnodes)
-                info += "Average in degree: %8.4f\n" % deg
-                deg = sum(d for n, d in G.out_degree()) / float(nnodes)
-                info += "Average out degree: %8.4f" % deg
-            else:
-                s = sum(dict(G.degree()).values())
-                info += "Average degree: %8.4f" % (float(s) / float(nnodes))
 
-    else:
-        if n not in G:
-            raise nx.NetworkXError("node %s not in graph" % (n,))
-        info += "Node % s has the following properties:\n" % n
-        info += "Degree: %d\n" % G.degree(n)
-        info += "Neighbors: "
-        info += ' '.join(str(nbr) for nbr in G.neighbors(n))
+    Returns
+    -------
+    info : str
+        A string containing the short summary
+
+    Raises
+    ------
+    NetworkXError
+        If n is not in the graph G
+
+    """
+    if n is None:
+        return str(G)
+    if n not in G:
+        raise nx.NetworkXError(f"node {n} not in graph")
+    info = ""  # append this all to a string
+    info += f"Node {n} has the following properties:\n"
+    info += f"Degree: {G.degree(n)}\n"
+    info += "Neighbors: "
+    info += " ".join(str(nbr) for nbr in G.neighbors(n))
     return info
 
 
@@ -618,8 +622,8 @@ def set_node_attributes(G, values, name=None):
         >>> bb = nx.betweenness_centrality(G)
         >>> isinstance(bb, dict)
         True
-        >>> nx.set_node_attributes(G, bb, 'betweenness')
-        >>> G.nodes[1]['betweenness']
+        >>> nx.set_node_attributes(G, bb, "betweenness")
+        >>> G.nodes[1]["betweenness"]
         1.0
 
     If you provide a list as the second argument, updates to the list
@@ -627,13 +631,13 @@ def set_node_attributes(G, values, name=None):
 
         >>> G = nx.path_graph(3)
         >>> labels = []
-        >>> nx.set_node_attributes(G, labels, 'labels')
-        >>> labels.append('foo')
-        >>> G.nodes[0]['labels']
+        >>> nx.set_node_attributes(G, labels, "labels")
+        >>> labels.append("foo")
+        >>> G.nodes[0]["labels"]
         ['foo']
-        >>> G.nodes[1]['labels']
+        >>> G.nodes[1]["labels"]
         ['foo']
-        >>> G.nodes[2]['labels']
+        >>> G.nodes[2]["labels"]
         ['foo']
 
     If you provide a dictionary of dictionaries as the second argument,
@@ -641,17 +645,28 @@ def set_node_attributes(G, values, name=None):
     dictionary of node attributes for that node::
 
         >>> G = nx.path_graph(3)
-        >>> attrs = {0: {'attr1': 20, 'attr2': 'nothing'}, 1: {'attr2': 3}}
+        >>> attrs = {0: {"attr1": 20, "attr2": "nothing"}, 1: {"attr2": 3}}
         >>> nx.set_node_attributes(G, attrs)
-        >>> G.nodes[0]['attr1']
+        >>> G.nodes[0]["attr1"]
         20
-        >>> G.nodes[0]['attr2']
+        >>> G.nodes[0]["attr2"]
         'nothing'
-        >>> G.nodes[1]['attr2']
+        >>> G.nodes[1]["attr2"]
         3
         >>> G.nodes[2]
         {}
 
+    Note that if the dictionary contains nodes that are not in `G`, the
+    values are silently ignored::
+
+        >>> G = nx.Graph()
+        >>> G.add_node(0)
+        >>> nx.set_node_attributes(G, {0: "red", 1: "blue"}, name="color")
+        >>> G.nodes[0]["color"]
+        'red'
+        >>> 1 in G.nodes
+        False
+
     """
     # Set node attributes based on type of `values`
     if name is not None:  # `values` must not be a dict of dict
@@ -689,8 +704,8 @@ def get_node_attributes(G, name):
     Examples
     --------
     >>> G = nx.Graph()
-    >>> G.add_nodes_from([1, 2, 3], color='red')
-    >>> color = nx.get_node_attributes(G, 'color')
+    >>> G.add_nodes_from([1, 2, 3], color="red")
+    >>> color = nx.get_node_attributes(G, "color")
     >>> color[1]
     'red'
     """
@@ -733,35 +748,42 @@ def set_edge_attributes(G, values, name=None):
 
         >>> G = nx.path_graph(3)
         >>> bb = nx.edge_betweenness_centrality(G, normalized=False)
-        >>> nx.set_edge_attributes(G, bb, 'betweenness')
-        >>> G.edges[1, 2]['betweenness']
+        >>> nx.set_edge_attributes(G, bb, "betweenness")
+        >>> G.edges[1, 2]["betweenness"]
         2.0
 
     If you provide a list as the second argument, updates to the list
     will be reflected in the edge attribute for each edge::
 
         >>> labels = []
-        >>> nx.set_edge_attributes(G, labels, 'labels')
-        >>> labels.append('foo')
-        >>> G.edges[0, 1]['labels']
+        >>> nx.set_edge_attributes(G, labels, "labels")
+        >>> labels.append("foo")
+        >>> G.edges[0, 1]["labels"]
         ['foo']
-        >>> G.edges[1, 2]['labels']
+        >>> G.edges[1, 2]["labels"]
         ['foo']
 
     If you provide a dictionary of dictionaries as the second argument,
     the entire dictionary will be used to update edge attributes::
 
         >>> G = nx.path_graph(3)
-        >>> attrs = {(0, 1): {'attr1': 20, 'attr2': 'nothing'},
-        ...          (1, 2): {'attr2': 3}}
+        >>> attrs = {(0, 1): {"attr1": 20, "attr2": "nothing"}, (1, 2): {"attr2": 3}}
         >>> nx.set_edge_attributes(G, attrs)
-        >>> G[0][1]['attr1']
+        >>> G[0][1]["attr1"]
         20
-        >>> G[0][1]['attr2']
+        >>> G[0][1]["attr2"]
         'nothing'
-        >>> G[1][2]['attr2']
+        >>> G[1][2]["attr2"]
         3
 
+    Note that if the dict contains edges that are not in `G`, they are
+    silently ignored::
+
+        >>> G = nx.Graph([(0, 1)])
+        >>> nx.set_edge_attributes(G, {(1, 2): {"weight": 2.0}})
+        >>> (1, 2) in G.edges()
+        False
+
     """
     if name is not None:
         # `values` does not contain attribute names
@@ -818,8 +840,8 @@ def get_edge_attributes(G, name):
     Examples
     --------
     >>> G = nx.Graph()
-    >>> nx.add_path(G, [1, 2, 3], color='red')
-    >>> color = nx.get_edge_attributes(G, 'color')
+    >>> nx.add_path(G, [1, 2, 3], color="red")
+    >>> color = nx.get_edge_attributes(G, "color")
     >>> color[(1, 2)]
     'red'
     """
@@ -900,7 +922,7 @@ def non_edges(graph):
                 yield (u, v)
 
 
-@not_implemented_for('directed')
+@not_implemented_for("directed")
 def common_neighbors(G, u, v):
     """Returns the common neighbors of two nodes in a graph.
 
@@ -929,16 +951,16 @@ def common_neighbors(G, u, v):
     [2, 3, 4]
     """
     if u not in G:
-        raise nx.NetworkXError('u is not in the graph.')
+        raise nx.NetworkXError("u is not in the graph.")
     if v not in G:
-        raise nx.NetworkXError('v is not in the graph.')
+        raise nx.NetworkXError("v is not in the graph.")
 
     # Return a generator explicitly instead of yielding so that the above
     # checks are executed eagerly.
     return (w for w in G[u] if w in G[v] and w not in (u, v))
 
 
-def is_weighted(G, edge=None, weight='weight'):
+def is_weighted(G, edge=None, weight="weight"):
     """Returns True if `G` has weighted edges.
 
     Parameters
@@ -980,7 +1002,7 @@ def is_weighted(G, edge=None, weight='weight'):
     if edge is not None:
         data = G.get_edge_data(*edge)
         if data is None:
-            msg = 'Edge {!r} does not exist.'.format(edge)
+            msg = f"Edge {edge!r} does not exist."
             raise nx.NetworkXError(msg)
         return weight in data
 
@@ -991,7 +1013,7 @@ def is_weighted(G, edge=None, weight='weight'):
     return all(weight in data for u, v, data in G.edges(data=True))
 
 
-def is_negatively_weighted(G, edge=None, weight='weight'):
+def is_negatively_weighted(G, edge=None, weight="weight"):
     """Returns True if `G` has negatively weighted edges.
 
     Parameters
@@ -1024,11 +1046,11 @@ def is_negatively_weighted(G, edge=None, weight='weight'):
     >>> G.add_edge(1, 2, weight=4)
     >>> nx.is_negatively_weighted(G, (1, 2))
     False
-    >>> G[2][4]['weight'] = -2
+    >>> G[2][4]["weight"] = -2
     >>> nx.is_negatively_weighted(G)
     True
     >>> G = nx.DiGraph()
-    >>> edges = [('0', '3', 3), ('0', '1', -5), ('1', '0', -2)]
+    >>> edges = [("0", "3", 3), ("0", "1", -5), ("1", "0", -2)]
     >>> G.add_weighted_edges_from(edges)
     >>> nx.is_negatively_weighted(G)
     True
@@ -1037,12 +1059,11 @@ def is_negatively_weighted(G, edge=None, weight='weight'):
     if edge is not None:
         data = G.get_edge_data(*edge)
         if data is None:
-            msg = 'Edge {!r} does not exist.'.format(edge)
+            msg = f"Edge {edge!r} does not exist."
             raise nx.NetworkXError(msg)
         return weight in data and data[weight] < 0
 
-    return any(weight in data and data[weight] < 0
-               for u, v, data in G.edges(data=True))
+    return any(weight in data and data[weight] < 0 for u, v, data in G.edges(data=True))
 
 
 def is_empty(G):
@@ -1085,7 +1106,7 @@ def nodes_with_selfloops(G):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> G.add_edge(1, 1)
     >>> G.add_edge(1, 2)
     >>> list(nx.nodes_with_selfloops(G))
@@ -1102,6 +1123,8 @@ def selfloop_edges(G, data=False, keys=False, default=None):
 
     Parameters
     ----------
+    G : graph
+        A NetworkX graph.
     data : string or bool, optional (default=False)
         Return selfloop edges as two tuples (u, v) (data=False)
         or three-tuples (u, v, datadict) (data=True)
@@ -1123,7 +1146,7 @@ def selfloop_edges(G, data=False, keys=False, default=None):
 
     Examples
     --------
-    >>> G = nx.MultiGraph()   # or Graph, DiGraph, MultiDiGraph, etc
+    >>> G = nx.MultiGraph()  # or Graph, DiGraph, MultiDiGraph, etc
     >>> ekey = G.add_edge(1, 1)
     >>> ekey = G.add_edge(1, 2)
     >>> list(nx.selfloop_edges(G))
@@ -1138,38 +1161,56 @@ def selfloop_edges(G, data=False, keys=False, default=None):
     if data is True:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k, d)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k, d in nbrs[n].items())
+                return (
+                    (n, n, k, d)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for k, d in nbrs[n].items()
+                )
             else:
-                return ((n, n, d)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n, d)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for d in nbrs[n].values()
+                )
         else:
             return ((n, n, nbrs[n]) for n, nbrs in G.adj.items() if n in nbrs)
     elif data is not False:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k, d.get(data, default))
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k, d in nbrs[n].items())
+                return (
+                    (n, n, k, d.get(data, default))
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for k, d in nbrs[n].items()
+                )
             else:
-                return ((n, n, d.get(data, default))
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n, d.get(data, default))
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for d in nbrs[n].values()
+                )
         else:
-            return ((n, n, nbrs[n].get(data, default))
-                    for n, nbrs in G.adj.items() if n in nbrs)
+            return (
+                (n, n, nbrs[n].get(data, default))
+                for n, nbrs in G.adj.items()
+                if n in nbrs
+            )
     else:
         if G.is_multigraph():
             if keys is True:
-                return ((n, n, k)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for k in nbrs[n])
+                return (
+                    (n, n, k) for n, nbrs in G.adj.items() if n in nbrs for k in nbrs[n]
+                )
             else:
-                return ((n, n)
-                        for n, nbrs in G.adj.items()
-                        if n in nbrs for d in nbrs[n].values())
+                return (
+                    (n, n)
+                    for n, nbrs in G.adj.items()
+                    if n in nbrs
+                    for i in range(len(nbrs[n]))  # for easy edge removal (#4068)
+                )
         else:
             return ((n, n) for n, nbrs in G.adj.items() if n in nbrs)
 
@@ -1190,10 +1231,71 @@ def number_of_selfloops(G):
 
     Examples
     --------
-    >>> G = nx.Graph()   # or DiGraph, MultiGraph, MultiDiGraph, etc
+    >>> G = nx.Graph()  # or DiGraph, MultiGraph, MultiDiGraph, etc
     >>> G.add_edge(1, 1)
     >>> G.add_edge(1, 2)
     >>> nx.number_of_selfloops(G)
     1
     """
     return sum(1 for _ in nx.selfloop_edges(G))
+
+
+def is_path(G, path):
+    """Returns whether or not the specified path exists
+
+    Parameters
+    ----------
+    G : graph
+        A NetworkX graph.
+
+    path: list
+        A list of node labels which defines the path to traverse
+
+    Returns
+    -------
+    isPath: bool
+        A boolean representing whether or not the path exists
+
+    """
+    for node, nbr in nx.utils.pairwise(path):
+        if nbr not in G[node]:
+            return False
+    return True
+
+
+def path_weight(G, path, weight):
+    """Returns total cost associated with specified path and weight
+
+    Parameters
+    ----------
+    G : graph
+        A NetworkX graph.
+
+    path: list
+        A list of node labels which defines the path to traverse
+
+    weight: string
+        A string indicating which edge attribute to use for path cost
+
+    Returns
+    -------
+    cost: int
+        A integer representing the total cost with respect to the
+        specified weight of the specified path
+
+    Raises
+    ------
+    NetworkXNoPath
+        If the specified edge does not exist.
+    """
+    multigraph = G.is_multigraph()
+    cost = 0
+
+    if not nx.is_path(G, path):
+        raise nx.NetworkXNoPath("path does not exist")
+    for node, nbr in nx.utils.pairwise(path):
+        if multigraph:
+            cost += min([v[weight] for v in G[node][nbr].values()])
+        else:
+            cost += G[node][nbr][weight]
+    return cost
