@@ -1,9 +1,3 @@
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
 """Functions for finding and manipulating cliques.
 
 Finding the largest clique in a graph is NP-complete problem, so most of
@@ -17,21 +11,26 @@ from collections import deque
 from itertools import chain
 from itertools import combinations
 from itertools import islice
-try:
-    from itertools import ifilter as filter
-except ImportError:
-    pass
 import networkx as nx
 from networkx.utils import not_implemented_for
-__author__ = """Dan Schult (dschult@colgate.edu)"""
-__all__ = ['find_cliques', 'find_cliques_recursive', 'make_max_clique_graph',
-           'make_clique_bipartite', 'graph_clique_number',
-           'graph_number_of_cliques', 'node_clique_number',
-           'number_of_cliques', 'cliques_containing_node',
-           'enumerate_all_cliques']
 
 
-@not_implemented_for('directed')
+__all__ = [
+    "find_cliques",
+    "find_cliques_recursive",
+    "make_max_clique_graph",
+    "make_clique_bipartite",
+    "graph_clique_number",
+    "graph_number_of_cliques",
+    "node_clique_number",
+    "number_of_cliques",
+    "cliques_containing_node",
+    "enumerate_all_cliques",
+    "max_weight_clique",
+]
+
+
+@not_implemented_for("directed")
 def enumerate_all_cliques(G):
     """Returns all cliques in an undirected graph.
 
@@ -94,12 +93,15 @@ def enumerate_all_cliques(G):
         yield base
         for i, u in enumerate(cnbrs):
             # Use generators to reduce memory consumption.
-            queue.append((chain(base, [u]),
-                          filter(nbrs[u].__contains__,
-                                 islice(cnbrs, i + 1, None))))
+            queue.append(
+                (
+                    chain(base, [u]),
+                    filter(nbrs[u].__contains__, islice(cnbrs, i + 1, None)),
+                )
+            )
 
 
-@not_implemented_for('directed')
+@not_implemented_for("directed")
 def find_cliques(G):
     """Returns all maximal cliques in an undirected graph.
 
@@ -132,9 +134,8 @@ def find_cliques(G):
     To obtain a list of all maximal cliques, use
     `list(find_cliques(G))`. However, be aware that in the worst-case,
     the length of this list can be exponential in the number of nodes in
-    the graph (for example, when the graph is the complete graph). This
-    function avoids storing all cliques in memory by only keeping
-    current candidate node lists in memory during its search.
+    the graph. This function avoids storing all cliques in memory by
+    only keeping current candidate node lists in memory during its search.
 
     This implementation is based on the algorithm published by Bron and
     Kerbosch (1973) [1]_, as adapted by Tomita, Tanaka and Takahashi
@@ -239,9 +240,8 @@ def find_cliques_recursive(G):
     To obtain a list of all maximal cliques, use
     `list(find_cliques_recursive(G))`. However, be aware that in the
     worst-case, the length of this list can be exponential in the number
-    of nodes in the graph (for example, when the graph is the complete
-    graph). This function avoids storing all cliques in memory by only
-    keeping current candidate node lists in memory during its search.
+    of nodes in the graph. This function avoids storing all cliques in memory
+    by only keeping current candidate node lists in memory during its search.
 
     This implementation is based on the algorithm published by Bron and
     Kerbosch (1973) [1]_, as adapted by Tomita, Tanaka and Takahashi
@@ -292,8 +292,7 @@ def find_cliques_recursive(G):
             else:
                 cand_q = cand & adj_q
                 if cand_q:
-                    for clique in expand(subg_q, cand_q):
-                        yield clique
+                    yield from expand(subg_q, cand_q)
             Q.pop()
 
     return expand(set(G), set(G))
@@ -421,10 +420,10 @@ def graph_clique_number(G, cliques=None):
     maximal cliques.
 
     """
-    if cliques is None:
-        cliques = find_cliques(G)
     if len(G.nodes) < 1:
         return 0
+    if cliques is None:
+        cliques = find_cliques(G)
     return max([len(c) for c in cliques] or [1])
 
 
@@ -459,7 +458,7 @@ def graph_number_of_cliques(G, cliques=None):
 
 
 def node_clique_number(G, nodes=None, cliques=None):
-    """ Returns the size of the largest maximal clique containing
+    """Returns the size of the largest maximal clique containing
     each given node.
 
     Returns a single or list depending on input nodes.
@@ -472,18 +471,18 @@ def node_clique_number(G, nodes=None, cliques=None):
                 d = {}
                 for n in nodes:
                     H = nx.ego_graph(G, n)
-                    d[n] = max((len(c) for c in find_cliques(H)))
+                    d[n] = max(len(c) for c in find_cliques(H))
             else:
                 H = nx.ego_graph(G, nodes)
-                d = max((len(c) for c in find_cliques(H)))
+                d = max(len(c) for c in find_cliques(H))
             return d
         # nodes is None--find all cliques
         cliques = list(find_cliques(G))
 
     if nodes is None:
-        nodes = list(G.nodes())   # none, get entire graph
+        nodes = list(G.nodes())  # none, get entire graph
 
-    if not isinstance(nodes, list):   # check for a list
+    if not isinstance(nodes, list):  # check for a list
         v = nodes
         # assume it is a single value
         d = max([len(c) for c in cliques if v in c])
@@ -519,9 +518,9 @@ def number_of_cliques(G, nodes=None, cliques=None):
         cliques = list(find_cliques(G))
 
     if nodes is None:
-        nodes = list(G.nodes())   # none, get entire graph
+        nodes = list(G.nodes())  # none, get entire graph
 
-    if not isinstance(nodes, list):   # check for a list
+    if not isinstance(nodes, list):  # check for a list
         v = nodes
         # assume it is a single value
         numcliq = len([1 for c in cliques if v in c])
@@ -542,9 +541,9 @@ def cliques_containing_node(G, nodes=None, cliques=None):
         cliques = list(find_cliques(G))
 
     if nodes is None:
-        nodes = list(G.nodes())   # none, get entire graph
+        nodes = list(G.nodes())  # none, get entire graph
 
-    if not isinstance(nodes, list):   # check for a list
+    if not isinstance(nodes, list):  # check for a list
         v = nodes
         # assume it is a single value
         vcliques = [c for c in cliques if v in c]
@@ -553,3 +552,160 @@ def cliques_containing_node(G, nodes=None, cliques=None):
         for v in nodes:
             vcliques[v] = [c for c in cliques if v in c]
     return vcliques
+
+
+class MaxWeightClique(object):
+    """A class for the maximum weight clique algorithm.
+
+    This class is a helper for the `max_weight_clique` function.  The class
+    should not normally be used directly.
+
+    Parameters
+    ----------
+    G : NetworkX graph
+        The undirected graph for which a maximum weight clique is sought
+    weight : string or None, optional (default='weight')
+        The node attribute that holds the integer value used as a weight.
+        If None, then each node has weight 1.
+
+    Attributes
+    ----------
+    G : NetworkX graph
+        The undirected graph for which a maximum weight clique is sought
+    node_weights: dict
+        The weight of each node
+    incumbent_nodes : list
+        The nodes of the incumbent clique (the best clique found so far)
+    incumbent_weight: int
+        The weight of the incumbent clique
+    """
+
+    def __init__(self, G, weight):
+        self.G = G
+        self.incumbent_nodes = []
+        self.incumbent_weight = 0
+
+        if weight is None:
+            self.node_weights = {v: 1 for v in G.nodes()}
+        else:
+            for v in G.nodes():
+                if weight not in G.nodes[v]:
+                    errmsg = f"Node {v!r} does not have the requested weight field."
+                    raise KeyError(errmsg)
+                if not isinstance(G.nodes[v][weight], int):
+                    errmsg = f"The {weight!r} field of node {v!r} is not an integer."
+                    raise ValueError(errmsg)
+            self.node_weights = {v: G.nodes[v][weight] for v in G.nodes()}
+
+    def update_incumbent_if_improved(self, C, C_weight):
+        """Update the incumbent if the node set C has greater weight.
+
+        C is assumed to be a clique.
+        """
+        if C_weight > self.incumbent_weight:
+            self.incumbent_nodes = C[:]
+            self.incumbent_weight = C_weight
+
+    def greedily_find_independent_set(self, P):
+        """Greedily find an independent set of nodes from a set of
+        nodes P."""
+        independent_set = []
+        P = P[:]
+        while P:
+            v = P[0]
+            independent_set.append(v)
+            P = [w for w in P if v != w and not self.G.has_edge(v, w)]
+        return independent_set
+
+    def find_branching_nodes(self, P, target):
+        """Find a set of nodes to branch on."""
+        residual_wt = {v: self.node_weights[v] for v in P}
+        total_wt = 0
+        P = P[:]
+        while P:
+            independent_set = self.greedily_find_independent_set(P)
+            min_wt_in_class = min(residual_wt[v] for v in independent_set)
+            total_wt += min_wt_in_class
+            if total_wt > target:
+                break
+            for v in independent_set:
+                residual_wt[v] -= min_wt_in_class
+            P = [v for v in P if residual_wt[v] != 0]
+        return P
+
+    def expand(self, C, C_weight, P):
+        """Look for the best clique that contains all the nodes in C and zero or
+        more of the nodes in P, backtracking if it can be shown that no such
+        clique has greater weight than the incumbent.
+        """
+        self.update_incumbent_if_improved(C, C_weight)
+        branching_nodes = self.find_branching_nodes(P, self.incumbent_weight - C_weight)
+        while branching_nodes:
+            v = branching_nodes.pop()
+            P.remove(v)
+            new_C = C + [v]
+            new_C_weight = C_weight + self.node_weights[v]
+            new_P = [w for w in P if self.G.has_edge(v, w)]
+            self.expand(new_C, new_C_weight, new_P)
+
+    def find_max_weight_clique(self):
+        """Find a maximum weight clique."""
+        # Sort nodes in reverse order of degree for speed
+        nodes = sorted(self.G.nodes(), key=lambda v: self.G.degree(v), reverse=True)
+        nodes = [v for v in nodes if self.node_weights[v] > 0]
+        self.expand([], 0, nodes)
+
+
+@not_implemented_for("directed")
+def max_weight_clique(G, weight="weight"):
+    """Find a maximum weight clique in G.
+
+    A *clique* in a graph is a set of nodes such that every two distinct nodes
+    are adjacent.  The *weight* of a clique is the sum of the weights of its
+    nodes.  A *maximum weight clique* of graph G is a clique C in G such that
+    no clique in G has weight greater than the weight of C.
+
+    Parameters
+    ----------
+    G : NetworkX graph
+        Undirected graph
+    weight : string or None, optional (default='weight')
+        The node attribute that holds the integer value used as a weight.
+        If None, then each node has weight 1.
+
+    Returns
+    -------
+    clique : list
+        the nodes of a maximum weight clique
+    weight : int
+        the weight of a maximum weight clique
+
+    Notes
+    -----
+    The implementation is recursive, and therefore it may run into recursion
+    depth issues if G contains a clique whose number of nodes is close to the
+    recursion depth limit.
+
+    At each search node, the algorithm greedily constructs a weighted
+    independent set cover of part of the graph in order to find a small set of
+    nodes on which to branch.  The algorithm is very similar to the algorithm
+    of Tavares et al. [1]_, other than the fact that the NetworkX version does
+    not use bitsets.  This style of algorithm for maximum weight clique (and
+    maximum weight independent set, which is the same problem but on the
+    complement graph) has a decades-long history.  See Algorithm B of Warren
+    and Hicks [2]_ and the references in that paper.
+
+    References
+    ----------
+    .. [1] Tavares, W.A., Neto, M.B.C., Rodrigues, C.D., Michelon, P.: Um
+           algoritmo de branch and bound para o problema da clique máxima
+           ponderada.  Proceedings of XLVII SBPO 1 (2015).
+
+    .. [2] Warrent, Jeffrey S, Hicks, Illya V.: Combinatorial Branch-and-Bound
+           for the Maximum Weight Independent Set Problem.  Technical Report,
+           Texas A&M University (2016).
+    """
+
+    mwc = MaxWeightClique(G, weight)
+    mwc.find_max_weight_clique()
+    return mwc.incumbent_nodes, mwc.incumbent_weight
