@@ -1,14 +1,3 @@
-#    Copyright (C) 2010-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Jon Olav Vik <jonovik@gmail.com>
-#          Dan Schult <dschult@colgate.edu>
-#          Aric Hagberg <hagberg@lanl.gov>
-#          Debsankha Manik <dmanik@nld.ds.mpg.de>
 """
 ========================
 Cycle finding algorithms
@@ -16,22 +5,23 @@ Cycle finding algorithms
 """
 
 from collections import defaultdict
-from itertools import tee
 
 import networkx as nx
 from networkx.utils import not_implemented_for, pairwise
 
 __all__ = [
-    'cycle_basis', 'simple_cycles',
-    'recursive_simple_cycles', 'find_cycle',
-    'minimum_cycle_basis',
+    "cycle_basis",
+    "simple_cycles",
+    "recursive_simple_cycles",
+    "find_cycle",
+    "minimum_cycle_basis",
 ]
 
 
-@not_implemented_for('directed')
-@not_implemented_for('multigraph')
+@not_implemented_for("directed")
+@not_implemented_for("multigraph")
 def cycle_basis(G, root=None):
-    """ Returns a list of cycles which form a basis for cycles of G.
+    """Returns a list of cycles which form a basis for cycles of G.
 
     A basis for cycles of a network is a minimal collection of
     cycles such that any cycle in the network can be written
@@ -84,11 +74,11 @@ def cycle_basis(G, root=None):
             z = stack.pop()  # use last-in so cycles easier to find
             zused = used[z]
             for nbr in G[z]:
-                if nbr not in used:   # new node
+                if nbr not in used:  # new node
                     pred[nbr] = z
                     stack.append(nbr)
-                    used[nbr] = set([z])
-                elif nbr == z:          # self loops
+                    used[nbr] = {z}
+                elif nbr == z:  # self loops
                     cycles.append([z])
                 elif nbr not in zused:  # found a cycle
                     pn = used[nbr]
@@ -105,7 +95,7 @@ def cycle_basis(G, root=None):
     return cycles
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def simple_cycles(G):
     """Find simple cycles (elementary circuits) of a directed graph.
 
@@ -166,8 +156,9 @@ def simple_cycles(G):
     --------
     cycle_basis
     """
+
     def _unblock(thisnode, blocked, B):
-        stack = set([thisnode])
+        stack = {thisnode}
         while stack:
             node = stack.pop()
             if node in blocked:
@@ -181,8 +172,7 @@ def simple_cycles(G):
     # Also we save the actual graph so we can mutate it. We only take the
     # edges because we do not want to copy edge and node attributes here.
     subG = type(G)(G.edges())
-    sccs = [scc for scc in nx.strongly_connected_components(subG)
-            if len(scc) > 1]
+    sccs = [scc for scc in nx.strongly_connected_components(subG) if len(scc) > 1]
 
     # Johnson's algorithm exclude self cycle edges like (v, v)
     # To be backward compatible, we record those cycles in advance
@@ -200,7 +190,7 @@ def simple_cycles(G):
         # Processing node runs "circuit" routine from recursive version
         path = [startnode]
         blocked = set()  # vertex: blocked from search?
-        closed = set()   # nodes involved in a cycle
+        closed = set()  # nodes involved in a cycle
         blocked.add(startnode)
         B = defaultdict(set)  # graph portions that yield no elementary circuit
         stack = [(startnode, list(sccG[startnode]))]  # sccG gives comp nbrs
@@ -211,7 +201,7 @@ def simple_cycles(G):
                 if nextnode == startnode:
                     yield path[:]
                     closed.update(path)
-#                        print "Found a cycle", path, closed
+                #                        print "Found a cycle", path, closed
                 elif nextnode not in blocked:
                     path.append(nextnode)
                     stack.append((nextnode, list(sccG[nextnode])))
@@ -227,15 +217,14 @@ def simple_cycles(G):
                         if thisnode not in B[nbr]:
                             B[nbr].add(thisnode)
                 stack.pop()
-#                assert path[-1] == thisnode
+                #                assert path[-1] == thisnode
                 path.pop()
         # done processing this node
         H = subG.subgraph(scc)  # make smaller to avoid work in SCC routine
-        sccs.extend(scc for scc in nx.strongly_connected_components(H)
-                    if len(scc) > 1)
+        sccs.extend(scc for scc in nx.strongly_connected_components(H) if len(scc) > 1)
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def recursive_simple_cycles(G):
     """Find simple cycles (elementary circuits) of a directed graph.
 
@@ -313,10 +302,10 @@ def recursive_simple_cycles(G):
         path.pop()  # remove thisnode from path
         return closed
 
-    path = []              # stack of nodes in current path
+    path = []  # stack of nodes in current path
     blocked = defaultdict(bool)  # vertex: blocked from search?
     B = defaultdict(list)  # graph portions that yield no elementary circuit
-    result = []            # list to accumulate the circuits found
+    result = []  # list to accumulate the circuits found
 
     # Johnson's algorithm exclude self cycle edges like (v, v)
     # To be backward compatible, we record those cycles in advance
@@ -331,8 +320,7 @@ def recursive_simple_cycles(G):
     ordering = dict(zip(G, range(len(G))))
     for s in ordering:
         # Build the subgraph induced by s and following nodes in the ordering
-        subgraph = G.subgraph(node for node in G
-                              if ordering[node] >= ordering[s])
+        subgraph = G.subgraph(node for node in G if ordering[node] >= ordering[s])
         # Find the strongly connected component in the subgraph
         # that contains the least node according to the ordering
         strongcomp = nx.strongly_connected_components(subgraph)
@@ -403,26 +391,32 @@ def find_cycle(G, source=None, orientation=None):
     This means that this DAG structure does not form a directed tree (which
     is also known as a polytree).
 
-    >>> import networkx as nx
     >>> G = nx.DiGraph([(0, 1), (0, 2), (1, 2)])
-    >>> try:
-    ...    nx.find_cycle(G, orientation='original')
-    ... except:
-    ...    pass
-    ...
-    >>> list(nx.find_cycle(G, orientation='ignore'))
+    >>> nx.find_cycle(G, orientation="original")
+    Traceback (most recent call last):
+        ...
+    networkx.exception.NetworkXNoCycle: No cycle found.
+    >>> list(nx.find_cycle(G, orientation="ignore"))
     [(0, 1, 'forward'), (1, 2, 'forward'), (0, 2, 'reverse')]
 
+    See Also
+    --------
+    simple_cycles
     """
-    if not G.is_directed() or orientation in (None, 'original'):
+    if not G.is_directed() or orientation in (None, "original"):
+
         def tailhead(edge):
             return edge[:2]
-    elif orientation == 'reverse':
+
+    elif orientation == "reverse":
+
         def tailhead(edge):
             return edge[1], edge[0]
-    elif orientation == 'ignore':
+
+    elif orientation == "ignore":
+
         def tailhead(edge):
-            if edge[-1] == 'reverse':
+            if edge[-1] == "reverse":
                 return edge[1], edge[0]
             return edge[:2]
 
@@ -487,8 +481,8 @@ def find_cycle(G, source=None, orientation=None):
             explored.update(seen)
 
     else:
-        assert(len(cycle) == 0)
-        raise nx.exception.NetworkXNoCycle('No cycle found.')
+        assert len(cycle) == 0
+        raise nx.exception.NetworkXNoCycle("No cycle found.")
 
     # We now have a list of edges which ends on a cycle.
     # So we need to remove from the beginning edges that are not relevant.
@@ -501,10 +495,10 @@ def find_cycle(G, source=None, orientation=None):
     return cycle[i:]
 
 
-@not_implemented_for('directed')
-@not_implemented_for('multigraph')
+@not_implemented_for("directed")
+@not_implemented_for("multigraph")
 def minimum_cycle_basis(G, weight=None):
-    """ Returns a minimum weight cycle basis for G
+    """Returns a minimum weight cycle basis for G
 
     Minimum weight means a cycle basis for which the total weight
     (length for unweighted graphs) of all the cycles is minimum.
@@ -523,10 +517,10 @@ def minimum_cycle_basis(G, weight=None):
 
     Examples
     --------
-    >>> G=nx.Graph()
-    >>> G.add_cycle([0,1,2,3])
-    >>> G.add_cycle([0,3,4,5])
-    >>> print(nx.minimum_cycle_basis(G))
+    >>> G = nx.Graph()
+    >>> nx.add_cycle(G, [0, 1, 2, 3])
+    >>> nx.add_cycle(G, [0, 3, 4, 5])
+    >>> print([sorted(c) for c in nx.minimum_cycle_basis(G)])
     [[0, 1, 2, 3], [0, 3, 4, 5]]
 
     References:
@@ -541,8 +535,10 @@ def minimum_cycle_basis(G, weight=None):
     simple_cycles, cycle_basis
     """
     # We first split the graph in commected subgraphs
-    return sum((_min_cycle_basis(c, weight) for c in
-                nx.connected_component_subgraphs(G)), [])
+    return sum(
+        (_min_cycle_basis(G.subgraph(c), weight) for c in nx.connected_components(G)),
+        [],
+    )
 
 
 def _min_cycle_basis(comp, weight):
@@ -550,14 +546,12 @@ def _min_cycle_basis(comp, weight):
     # We  extract the edges not in a spanning tree. We do not really need a
     # *minimum* spanning tree. That is why we call the next function with
     # weight=None. Depending on implementation, it may be faster as well
-    spanning_tree_edges = list(nx.minimum_spanning_edges(comp, weight=None,
-                                                         data=False))
-    edges_excl = [frozenset(e) for e in comp.edges()
-                  if e not in spanning_tree_edges]
+    spanning_tree_edges = list(nx.minimum_spanning_edges(comp, weight=None, data=False))
+    edges_excl = [frozenset(e) for e in comp.edges() if e not in spanning_tree_edges]
     N = len(edges_excl)
 
     # We maintain a set of vectors orthogonal to sofar found cycles
-    set_orth = [set([edge]) for edge in edges_excl]
+    set_orth = [{edge} for edge in edges_excl]
     for k in range(N):
         # kth cycle is "parallel" to kth vector in set_orth
         new_cycle = _min_cycle(comp, set_orth[k], weight=weight)
@@ -565,8 +559,10 @@ def _min_cycle_basis(comp, weight):
         # now update set_orth so that k+1,k+2... th elements are
         # orthogonal to the newly found cycle, as per [p. 336, 1]
         base = set_orth[k]
-        set_orth[k + 1:] = [orth ^ base if len(orth & new_cycle) % 2 else orth
-                            for orth in set_orth[k + 1:]]
+        set_orth[k + 1 :] = [
+            orth ^ base if len(orth & new_cycle) % 2 else orth
+            for orth in set_orth[k + 1 :]
+        ]
     return cb
 
 
@@ -589,23 +585,25 @@ def _min_cycle(G, orth, weight=None):
         edge_w = data.get(weight, 1)
         if frozenset((u, v)) in orth:
             T.add_edges_from(
-                [(uidx, nnodes + vidx), (nnodes + uidx, vidx)], weight=edge_w)
+                [(uidx, nnodes + vidx), (nnodes + uidx, vidx)], weight=edge_w
+            )
         else:
             T.add_edges_from(
-                [(uidx, vidx), (nnodes + uidx, nnodes + vidx)], weight=edge_w)
+                [(uidx, vidx), (nnodes + uidx, nnodes + vidx)], weight=edge_w
+            )
 
     all_shortest_pathlens = dict(nx.shortest_path_length(T, weight=weight))
-    cross_paths_w_lens = {n: all_shortest_pathlens[n][nnodes + n]
-                          for n in range(nnodes)}
+    cross_paths_w_lens = {
+        n: all_shortest_pathlens[n][nnodes + n] for n in range(nnodes)
+    }
 
     # Now compute shortest paths in T, which translates to cyles in G
     start = min(cross_paths_w_lens, key=cross_paths_w_lens.get)
     end = nnodes + start
-    min_path = nx.shortest_path(T, source=start, target=end, weight='weight')
+    min_path = nx.shortest_path(T, source=start, target=end, weight="weight")
 
     # Now we obtain the actual path, re-map nodes in T to those in G
-    min_path_nodes = [node if node < nnodes else node - nnodes
-                      for node in min_path]
+    min_path_nodes = [node if node < nnodes else node - nnodes for node in min_path]
     # Now remove the edges that occur two times
     mcycle_pruned = _path_to_cycle(min_path_nodes)
 
