@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """
 ========
 Football
@@ -12,19 +11,8 @@ into a NetworkX graph.
 Requires Internet connection to download the URL
 http://www-personal.umich.edu/~mejn/netdata/football.zip
 """
-# Author: Aric Hagberg (hagberg@lanl.gov)
 
-#    Copyright (C) 2007-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-
-try:  # Python 3.x
-    import urllib.request as urllib
-except ImportError:  # Python 2.x
-    import urllib
+import urllib.request
 import io
 import zipfile
 
@@ -33,28 +21,29 @@ import networkx as nx
 
 url = "http://www-personal.umich.edu/~mejn/netdata/football.zip"
 
-sock = urllib.urlopen(url)  # open URL
+sock = urllib.request.urlopen(url)  # open URL
 s = io.BytesIO(sock.read())  # read into BytesIO "file"
 sock.close()
 
 zf = zipfile.ZipFile(s)  # zipfile object
-txt = zf.read('football.txt').decode()  # read info file
-gml = zf.read('football.gml').decode()  # read gml data
+txt = zf.read("football.txt").decode()  # read info file
+gml = zf.read("football.gml").decode()  # read gml data
 # throw away bogus first line with # from mejn files
-gml = gml.split('\n')[1:]
+gml = gml.split("\n")[1:]
 G = nx.parse_gml(gml)  # parse gml data
 
 print(txt)
 # print degree for each team - number of games
 for n, d in G.degree():
-    print('%s %d' % (n, d))
+    print(f"{n:20} {d:2}")
 
 options = {
-    'node_color': 'black',
-    'node_size': 50,
-    'line_color': 'grey',
-    'linewidths': 0,
-    'width': 0.1,
+    "node_color": "black",
+    "node_size": 50,
+    "linewidths": 0,
+    "width": 0.1,
 }
-nx.draw(G, **options)
+
+pos = nx.spring_layout(G, seed=1969)  # Seed for reproducible layout
+nx.draw(G, pos, **options)
 plt.show()
